"""
文件名解析服务
"""

from typing import Dict, List, Optional, Tuple

from ..exceptions import APIError


class NameResolver:
    """文件名到ID的解析器"""

    def __init__(self, file_service):
        self.file_service = file_service
        self._cache = {}  # 缓存文件列表
        self._cache_folder_id = None
        self._name_cache = {}  # 缓存文件ID到真实名称的映射

    def resolve_path(self, path: str, current_folder_id: str = "0") -> Tuple[str, str]:
        """
        解析路径到文件ID和类型

        Args:
            path: 文件路径，支持以下格式：
                  - "文件名.txt" (在当前目录查找)
                  - "/文件夹/文件名.txt" (从根目录开始的绝对路径)
                  - "文件夹/文件名.txt" (相对路径)
                  - "文件夹名/" (文件夹，末尾带/)
            current_folder_id: 当前文件夹ID

        Returns:
            (file_id, file_type) - file_type为'file'或'folder'
        """
        if not path:
            raise ValueError("路径不能为空")

        # 处理绝对路径
        if path.startswith('/'):
            current_folder_id = "0"
            path = path[1:]  # 移除开头的/

        # 如果路径为空（只有/），返回根目录
        if not path:
            return "0", "folder"

        # 分割路径
        parts = [p for p in path.split('/') if p]

        # 逐级解析路径
        for i, part in enumerate(parts):
            is_last = (i == len(parts) - 1)

            # 如果是最后一部分且以/结尾，说明是文件夹
            if is_last and path.endswith('/'):
                file_id = self._find_in_folder(part, current_folder_id, 'folder')
                return file_id, 'folder'

            # 查找当前部分
            if is_last:
                # 最后一部分，可能是文件或文件夹
                try:
                    # 先尝试作为文件查找
                    file_id = self._find_in_folder(part, current_folder_id, 'file')
                    return file_id, 'file'
                except APIError:
                    # 如果找不到文件，尝试作为文件夹
                    file_id = self._find_in_folder(part, current_folder_id, 'folder')
                    return file_id, 'folder'
            else:
                # 中间部分，必须是文件夹
                current_folder_id = self._find_in_folder(part, current_folder_id, 'folder')

        raise APIError(f"无法解析路径: {path}")

    def _find_in_folder(self, name: str, folder_id: str, expected_type: Optional[str] = None) -> str:
        """
        在指定文件夹中查找文件或文件夹

        Args:
            name: 文件/文件夹名称
            folder_id: 文件夹ID
            expected_type: 期望的类型 ('file', 'folder', None表示任意)

        Returns:
            文件ID
        """
        # 总是刷新缓存以确保数据最新
        self._refresh_cache(folder_id)

        # 在缓存中查找
        for file_info in self._cache.get(folder_id, []):
            file_name = file_info.get('file_name', '')
            file_type = 'folder' if file_info.get('file_type') == 0 else 'file'
            file_id = file_info.get('fid', '')

            if file_name == name:
                if expected_type is None or file_type == expected_type:
                    # 缓存真实的文件名
                    self._name_cache[file_id] = file_name
                    return file_id

        # 如果没找到，抛出异常
        type_desc = f"{expected_type}类型的" if expected_type else ""
        raise APIError(f"在文件夹中找不到{type_desc}文件: {name}")

    def _refresh_cache(self, folder_id: str):
        """刷新指定文件夹的缓存"""
        try:
            # 获取文件夹内容
            result = self.file_service.list_files(folder_id, size=1000)
            file_list = result.get('data', {}).get('list', [])

            # 更新缓存
            self._cache[folder_id] = file_list
            self._cache_folder_id = folder_id

        except Exception as e:
            raise APIError(f"无法获取文件夹内容: {e}")

    def resolve_multiple_paths(self, paths: List[str], current_folder_id: str = "0") -> List[Tuple[str, str, str]]:
        """
        解析多个路径

        Args:
            paths: 路径列表
            current_folder_id: 当前文件夹ID

        Returns:
            [(file_id, file_type, original_path), ...] 列表
        """
        results = []
        for path in paths:
            try:
                file_id, file_type = self.resolve_path(path, current_folder_id)
                results.append((file_id, file_type, path))
            except Exception as e:
                raise APIError(f"解析路径 '{path}' 失败: {e}")

        return results

    def get_file_info_by_name(self, name: str, folder_id: str = "0") -> Dict:
        """
        根据文件名获取文件信息

        Args:
            name: 文件名
            folder_id: 文件夹ID

        Returns:
            文件信息字典
        """
        file_id = self._find_in_folder(name, folder_id)
        return self.file_service.get_file_info(file_id)

    def list_folder_contents(self, folder_id: str = "0") -> List[str]:
        """
        列出文件夹内容的名称列表

        Args:
            folder_id: 文件夹ID

        Returns:
            文件名列表
        """
        if self._cache_folder_id != folder_id:
            self._refresh_cache(folder_id)

        names = []
        for file_info in self._cache.get(folder_id, []):
            file_name = file_info.get('file_name', '')
            file_type = file_info.get('file_type', 1)

            # 文件夹名称后面加/
            if file_type == 0:
                names.append(f"{file_name}/")
            else:
                names.append(file_name)

        return sorted(names)

    def get_real_name(self, file_id: str) -> Optional[str]:
        """
        获取文件的真实名称（从列表缓存中获取）

        Args:
            file_id: 文件ID

        Returns:
            真实的文件名，如果没有缓存则返回None
        """
        return self._name_cache.get(file_id)

    def clear_cache(self):
        """清除缓存"""
        self._cache.clear()
        self._cache_folder_id = None
        self._name_cache.clear()



"""
批量分享命令
"""

from typing import List, Optional

import typer
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.table import Table

from ..utils import (get_client, handle_api_error, print_error, print_info,
                     print_success, print_warning)


def batch_share(
    output: Optional[str] = typer.Option(None, "--output", "-o", help="CSV输出文件名"),
    exclude: Optional[List[str]] = typer.Option(["来自：分享"], "--exclude", "-e", help="排除的目录名称模式"),
    dry_run: bool = typer.Option(False, "--dry-run", help="只扫描目录，不创建分享")
):
    """批量分享三级目录下的所有目标目录"""
    console = Console()

    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            from ...services.batch_share_service import BatchShareService

            batch_service = BatchShareService(client.api_client)

            print_info("🔍 开始扫描网盘目录结构...")

            # 显示排除模式
            if exclude:
                print_info(f"排除目录模式: {', '.join(exclude)}")

            # 收集目标目录
            with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                console=console
            ) as progress:
                task = progress.add_task("正在扫描目录...", total=None)
                target_directories = batch_service.collect_target_directories(exclude)
                progress.update(task, description=f"找到 {len(target_directories)} 个目标目录")

            if not target_directories:
                print_warning("没有找到任何需要分享的目标目录")
                return

            # 显示目录结构预览
            print_success(f"✅ 找到 {len(target_directories)} 个目标目录")

            # 创建目录预览表格
            table = Table(title="目标目录预览")
            table.add_column("序号", style="cyan", width=4)
            table.add_column("二级目录", style="blue", width=12)
            table.add_column("三级目录", style="green", width=12)
            table.add_column("完整路径", style="dim", width=50)

            # 显示前20个目录作为预览
            preview_count = min(20, len(target_directories))
            for i, target_dir in enumerate(target_directories[:preview_count], 1):
                table.add_row(
                    str(i),
                    target_dir['second_level'],
                    target_dir['third_level'],
                    target_dir['full_path']
                )

            console.print(table)

            if len(target_directories) > preview_count:
                print_info(f"... 还有 {len(target_directories) - preview_count} 个目录")

            # 如果是dry run模式，只显示预览
            if dry_run:
                print_info("🔍 Dry run 模式：仅扫描，不创建分享")
                return

            # 确认是否继续
            if len(target_directories) > 10:
                confirm = typer.confirm(f"确定要为这 {len(target_directories)} 个目录创建分享链接吗？")
                if not confirm:
                    print_info("操作已取消")
                    return

            print_info("📤 开始批量创建分享链接...")

            # 批量创建分享
            with Progress(console=console) as progress:
                task = progress.add_task("创建分享链接...", total=len(target_directories))

                share_results = []
                for i, target_dir in enumerate(target_directories):
                    try:
                        progress.update(task, description=f"正在创建: {target_dir['name']}")

                        # 创建分享
                        share_result = client.shares.create_share(
                            file_ids=[target_dir['fid']],
                            title=target_dir['name'],
                            expire_days=0,  # 永久
                            password=None   # 无密码
                        )

                        if share_result:
                            from datetime import datetime
                            share_info = {
                                'target_directory': target_dir,
                                'share_result': share_result,
                                'share_title': target_dir['name'],
                                'share_url': share_result.get('share_url', ''),
                                'created_time': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                                'success': True
                            }
                            share_results.append(share_info)
                        else:
                            from datetime import datetime
                            share_info = {
                                'target_directory': target_dir,
                                'share_title': target_dir['name'],
                                'share_url': '',
                                'created_time': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                                'success': False,
                                'error': '分享创建失败'
                            }
                            share_results.append(share_info)

                    except Exception as e:
                        from datetime import datetime
                        share_info = {
                            'target_directory': target_dir,
                            'share_title': target_dir['name'],
                            'share_url': '',
                            'created_time': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                            'success': False,
                            'error': str(e)
                        }
                        share_results.append(share_info)

                    progress.advance(task)

            # 统计结果
            successful = sum(1 for result in share_results if result['success'])
            failed = len(share_results) - successful

            print_success(f"✅ 批量分享完成!")
            print_info(f"   总计: {len(share_results)} 个目录")
            print_info(f"   成功: {successful} 个")
            if failed > 0:
                print_warning(f"   失败: {failed} 个")

            # 导出CSV
            csv_path = batch_service.export_to_csv(share_results, output)
            print_success(f"📊 CSV文件已保存: {csv_path}")

            # 显示成功的分享链接（前10个）
            successful_shares = [r for r in share_results if r['success']]
            if successful_shares:
                print_info("📋 成功创建的分享链接:")
                result_table = Table()
                result_table.add_column("目录", style="cyan", width=20)
                result_table.add_column("分享链接", style="green", width=50)

                for result in successful_shares[:10]:  # 只显示前10个
                    result_table.add_row(
                        result['share_title'],
                        result['share_url']
                    )

                console.print(result_table)

                if len(successful_shares) > 10:
                    print_info(f"... 还有 {len(successful_shares) - 10} 个分享链接，详见CSV文件")

            # 显示失败的目录
            failed_shares = [r for r in share_results if not r['success']]
            if failed_shares:
                print_warning("❌ 创建失败的目录:")
                for result in failed_shares:
                    error_msg = result.get('error', '未知错误')
                    print_warning(f"   {result['share_title']}: {error_msg}")

    except Exception as e:
        handle_api_error(e, "批量分享")
        raise typer.Exit(1)


def list_structure(
    level: int = typer.Option(3, "--level", "-l", help="显示目录层级深度 (1-4)"),
    exclude: Optional[List[str]] = typer.Option(["来自：分享"], "--exclude", "-e", help="排除的目录名称模式")
):
    """查看网盘目录结构"""
    console = Console()

    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            from ...services.batch_share_service import BatchShareService

            batch_service = BatchShareService(client.api_client)

            print_info(f"🔍 扫描网盘目录结构 (深度: {level} 级)...")

            if exclude:
                print_info(f"排除目录模式: {', '.join(exclude)}")

            # 显示目录结构
            if level >= 4:
                target_directories = batch_service.collect_target_directories(exclude)

                if target_directories:
                    # 按二级目录分组显示
                    from collections import defaultdict
                    grouped = defaultdict(lambda: defaultdict(list))

                    for target_dir in target_directories:
                        second = target_dir['second_level']
                        third = target_dir['third_level']
                        grouped[second][third].append(target_dir['name'])

                    print_success(f"✅ 找到 {len(target_directories)} 个目标目录")

                    for second_name, third_dirs in grouped.items():
                        print_info(f"\n📁 {second_name}/")
                        for third_name, targets in third_dirs.items():
                            print_info(f"  📁 {third_name}/")
                            for target_name in targets:
                                print_info(f"    📂 {target_name}")
                else:
                    print_warning("没有找到任何目标目录")
            else:
                print_info("此功能需要level=4来显示完整的四级目录结构")

    except Exception as e:
        handle_api_error(e, "查看目录结构")
        raise typer.Exit(1)



"""
基于API的登录模块
"""

import json
import sys
import threading
import time
import uuid
from typing import Dict, Optional, Tuple

import httpx

from ..config import get_config_dir
from ..exceptions import AuthenticationError
from ..utils.logger import get_logger
from ..utils.qr_code import display_qr_from_url


class APILogin:
    """基于API的登录管理器"""

    def __init__(self, timeout: int = 300):
        """
        初始化API登录

        Args:
            timeout: 登录超时时间（秒）
        """
        self.timeout = timeout
        self.config_dir = get_config_dir()
        self.client = httpx.Client(timeout=30.0)
        self.logger = get_logger(__name__)

        # 倒计时相关
        self._countdown_thread = None
        self._stop_countdown = False

        # 设置基本headers
        self.client.headers.update({
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/139.0.0.0 Safari/537.36',
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'zh-CN,zh;q=0.9',
            'Cache-Control': 'no-cache',
            'Pragma': 'no-cache',
            'Sec-Ch-Ua': '"Not;A=Brand";v="99", "Google Chrome";v="139", "Chromium";v="139"',
            'Sec-Ch-Ua-Mobile': '?0',
            'Sec-Ch-Ua-Platform': '"macOS"',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-site'
        })

        # 确保配置目录存在
        self.config_dir.mkdir(parents=True, exist_ok=True)

    def _show_countdown(self, total_seconds: int):
        """
        显示倒计时

        Args:
            total_seconds: 总倒计时秒数
        """
        def countdown():
            remaining = total_seconds - 1

            while remaining > 0 and not self._stop_countdown:
                minutes = remaining // 60
                seconds = remaining % 60

                # 清除当前行并显示倒计时
                sys.stdout.write(f"\r⏰ 二维码有效期剩余: {minutes:02d}:{seconds:02d}")
                sys.stdout.flush()

                time.sleep(1)
                remaining -= 1

            if not self._stop_countdown:
                sys.stdout.write("\r⏰ 二维码已过期，请重新获取\n")
                sys.stdout.flush()

        self._countdown_thread = threading.Thread(target=countdown, daemon=True)
        self._countdown_thread.start()

    def _stop_countdown_display(self):
        """停止倒计时显示"""
        self._stop_countdown = True
        if self._countdown_thread and self._countdown_thread.is_alive():
            self._countdown_thread.join(timeout=1)
        # 清除倒计时行
        sys.stdout.write("\r" + " " * 50 + "\r")
        sys.stdout.flush()

    def get_qr_code(self) -> Tuple[str, str]:
        """
        获取登录二维码

        Returns:
            (qr_token, qr_url) 二维码token和URL
        """
        try:
            # 使用发现的真实API
            api_url = 'https://uop.quark.cn/cas/ajax/getTokenForQrcodeLogin'

            # 生成随机request_id
            request_id = str(uuid.uuid4())

            params = {
                'client_id': '532',
                'v': '1.2',
                'request_id': request_id
            }

            self.logger.debug(f"获取二维码token: {api_url}")
            response = self.client.get(api_url, params=params)

            if response.status_code == 200:
                data = response.json()

                if data.get('status') == 2000000:
                    # 成功获取token
                    token = data.get('data', {}).get('members', {}).get('token')
                    if token:
                        self.logger.debug(f"获取到二维码token: {token}")

                        # 构造真实的二维码URL
                        # 基于实际抓包分析的URL格式
                        qr_url = self._build_qr_url(token)

                        return token, qr_url
                    else:
                        raise AuthenticationError("响应中未找到token")
                else:
                    raise AuthenticationError(f"获取token失败: {data.get('message', '未知错误')}")
            else:
                raise AuthenticationError(f"API请求失败: {response.status_code}")

        except Exception as e:
            if isinstance(e, AuthenticationError):
                raise
            raise AuthenticationError(f"获取二维码失败: {e}")

    def _build_qr_url(self, token: str) -> str:
        """
        构造二维码URL

        Args:
            token: 从API获取的token

        Returns:
            完整的二维码URL
        """
        # 基于实际抓包分析的URL格式
        base_url = "https://su.quark.cn/4_eMHBJ"

        # 构造查询参数
        params = {
            'token': token,
            'client_id': '532',
            'ssb': 'weblogin',
            'uc_param_str': '',
            'uc_biz_str': 'S:custom|OPT:SAREA@0|OPT:IMMERSIVE@1|OPT:BACK_BTN_STYLE@0'
        }

        # 构造完整URL
        import urllib.parse
        query_string = urllib.parse.urlencode(params)
        qr_url = f"{base_url}?{query_string}"

        self.logger.debug(f"构造二维码URL: {qr_url}")
        return qr_url

    def _is_valid_qr_response(self, data: Dict) -> bool:
        """检查响应是否包含有效的二维码数据"""
        if not isinstance(data, dict):
            return False

        # 检查常见的二维码字段
        qr_fields = ['qr_code', 'qrcode', 'qr_url', 'qrUrl', 'code_url', 'url', 'token', 'ticket']
        return any(field in data for field in qr_fields)

    def _process_qr_response(self, data: Dict) -> Tuple[str, str, str]:
        """处理二维码响应数据"""
        # 提取二维码相关信息
        qr_token = None
        qr_url = None

        # 尝试不同的字段名
        for token_field in ['token', 'ticket', 'qr_token', 'id']:
            if token_field in data:
                qr_token = data[token_field]
                break

        for url_field in ['qr_code', 'qrcode', 'qr_url', 'qrUrl', 'code_url', 'url']:
            if url_field in data:
                qr_url = data[url_field]
                break

        if not qr_url:
            raise AuthenticationError("响应中未找到二维码URL")

        return qr_token or str(uuid.uuid4()), qr_url

    def _fallback_get_qr_code(self) -> Tuple[str, str]:
        """备用方案：访问登录页面获取二维码"""
        try:
            # 访问夸克网盘登录页面
            login_urls = [
                'https://pan.quark.cn/account/login',
                'https://account.quark.cn/passport/login',
                'https://passport.quark.cn/login'
            ]

            for login_url in login_urls:
                try:
                    self.logger.info(f"访问登录页面: {login_url}")
                    response = self.client.get(login_url)

                    if response.status_code == 200:
                        # 在页面中查找二维码相关的API调用
                        html = response.text

                        # 查找可能的二维码API URL
                        import re
                        api_patterns = [
                            r'["\']([^"\']*qr[^"\']*)["\']',
                            r'["\']([^"\']*code[^"\']*)["\']',
                            r'url\s*:\s*["\']([^"\']*)["\']'
                        ]

                        for pattern in api_patterns:
                            matches = re.findall(pattern, html, re.IGNORECASE)
                            for match in matches:
                                if 'http' in match and ('qr' in match.lower() or 'code' in match.lower()):
                                    self.logger.info(f"找到可能的二维码API: {match}")
                                    # 尝试调用这个API
                                    try:
                                        api_response = self.client.get(match)
                                        if api_response.status_code == 200:
                                            data = api_response.json()
                                            if self._is_valid_qr_response(data):
                                                return self._process_qr_response(data)
                                    except:
                                        continue

                except Exception as e:
                    self.logger.debug(f"访问 {login_url} 失败: {e}")
                    continue

            raise AuthenticationError("无法通过任何方式获取二维码")

        except Exception as e:
            raise AuthenticationError(f"备用获取二维码方案失败: {e}")

    def check_login_status(self, qr_token: str) -> Optional[Dict]:
        """
        检查登录状态

        Args:
            qr_token: 二维码token

        Returns:
            登录成功时返回用户信息，否则返回None
        """
        try:
            # 使用发现的真实API
            api_url = 'https://uop.quark.cn/cas/ajax/getServiceTicketByQrcodeToken'

            # 生成随机request_id
            request_id = str(uuid.uuid4())

            params = {
                'client_id': '532',
                'v': '1.2',
                'token': qr_token,
                'request_id': request_id
            }

            self.logger.debug(f"检查登录状态: {api_url}")
            self.logger.debug(f"请求参数: {params}")

            response = self.client.get(api_url, params=params)
            self.logger.debug(f"响应状态码: {response.status_code}")

            if response.status_code == 200:
                data = response.json()
                self.logger.debug(f"API响应: {data}")

                # 检查登录状态
                if self._is_login_success(data):
                    self.logger.debug("登录成功检测通过")
                    return data
                elif self._is_login_failed(data):
                    self.logger.debug("登录失败检测通过")
                    return None
                else:
                    # 还在等待扫码
                    self.logger.debug("仍在等待扫码")
                    return None
            else:
                self.logger.error(f"API请求失败: {response.status_code}")

            return None

        except Exception as e:
            self.logger.error(f"检查登录状态失败: {e}")
            return None

    def _is_login_success(self, data: Dict) -> bool:
        """检查是否登录成功"""
        if not isinstance(data, dict):
            return False

        # 根据实际API响应检查成功标志
        status = data.get('status')
        message = data.get('message', '')

        # 登录成功的标志：status=2000000, message="ok", 且有data.members.service_ticket
        if (status == 2000000 and
            message == "ok" and
                data.get('data', {}).get('members', {}).get('service_ticket')):
            return True

        return False

    def _is_login_failed(self, data: Dict) -> bool:
        """检查是否登录失败"""
        if not isinstance(data, dict):
            return False

        # 检查失败标志
        status = data.get('status')
        message = data.get('message', '')

        # 明确的失败状态（不包括50004001，那是等待状态）
        fail_indicators = [
            status in [50004002, 50004003, 50004004],  # 明确的失败状态码
            'expired' in message.lower(),
            'failed' in message.lower(),
            'error' in message.lower(),
            'timeout' in message.lower(),
            'invalid' in message.lower()
        ]

        return any(fail_indicators)

    def wait_for_login(self, qr_token: str) -> bool:
        """
        等待用户扫码登录

        Args:
            qr_token: 二维码token

        Returns:
            登录是否成功
        """
        start_time = time.time()
        check_count = 0

        self.logger.debug(f"开始等待登录，超时时间: {self.timeout}秒")

        # 启动倒计时显示
        self._show_countdown(self.timeout)

        try:
            while time.time() - start_time < self.timeout:
                try:
                    check_count += 1
                    elapsed = int(time.time() - start_time)
                    self.logger.debug(f"第{check_count}次检查登录状态 (已等待{elapsed}秒)...")

                    result = self.check_login_status(qr_token)

                    if result is not None:
                        self.logger.debug(f"收到API响应: {result}")

                        if self._is_login_success(result):
                            self._stop_countdown_display()
                            self.logger.info("登录成功")
                            # 登录成功，保存cookies
                            self._save_login_result(result)
                            return True
                        elif self._is_login_failed(result):
                            self._stop_countdown_display()
                            self.logger.error("登录失败")
                            return False
                        else:
                            self.logger.debug("登录状态未知，继续等待...")
                    else:
                        self.logger.debug("等待扫码中...")

                    # 等待一段时间后再次检查
                    time.sleep(2)

                except Exception as e:
                    self.logger.error(f"等待登录时出错: {e}")
                    time.sleep(2)

            self._stop_countdown_display()
            self.logger.error(f"登录超时")
            return False

        finally:
            # 确保倒计时停止
            self._stop_countdown_display()

    def _save_login_result(self, result: Dict):
        """保存登录结果"""
        try:
            # 保存原始结果
            result_file = self.config_dir / "login_result.json"
            with open(result_file, 'w', encoding='utf-8') as f:
                json.dump(result, f, ensure_ascii=False, indent=2)

            self.logger.debug(f"登录结果已保存到: {result_file}")

            # 提取登录token (service ticket)
            service_ticket = result.get('data', {}).get('members', {}).get('service_ticket')
            if service_ticket:
                self.logger.debug(f"获取到service ticket: {service_ticket}")

                # 使用service ticket获取用户信息和Cookie
                self._get_user_info_and_cookies(service_ticket)

        except Exception as e:
            self.logger.error(f"保存登录结果失败: {e}")

    def _get_user_info_and_cookies(self, service_ticket: str):
        """
        使用service ticket获取用户信息和设置Cookie

        Args:
            service_ticket: 登录成功后获取的service ticket
        """
        try:
            # 调用用户信息API，这会设置登录Cookie
            api_url = 'https://pan.quark.cn/account/info'
            params = {
                'st': service_ticket,
                'lw': 'scan'  # 登录方式为扫码
            }

            self.logger.debug(f"获取用户信息: {api_url}")
            response = self.client.get(api_url, params=params)

            if response.status_code == 200:
                user_info = response.json()
                self.logger.debug("用户信息获取成功")

                # 保存用户信息
                user_info_file = self.config_dir / "user_info.json"
                with open(user_info_file, 'w', encoding='utf-8') as f:
                    json.dump(user_info, f, ensure_ascii=False, indent=2)

                self.logger.debug(f"用户信息已保存到: {user_info_file}")

                # 此时Cookie应该已经被自动设置到client中
                self.logger.debug("登录Cookie已设置")

            else:
                raise Exception(f"获取用户信息失败: {response.status_code}")

        except Exception as e:
            self.logger.error(f"获取用户信息失败: {e}")
            raise

    def login(self) -> str:
        """
        执行API登录流程

        Returns:
            Cookie字符串
        """
        try:
            # 获取二维码
            qr_token, qr_url = self.get_qr_code()

            print("请使用夸克APP扫描二维码登录...")

            # 显示二维码 - 直接从URL生成ASCII二维码
            try:
                display_qr_from_url(qr_url)
            except Exception as e:
                self.logger.warning(f"显示ASCII二维码失败: {e}")
                print(f"请在浏览器中打开: {qr_url}")

            # 等待登录
            if self.wait_for_login(qr_token):
                # 从client中提取cookies
                cookies = []
                for cookie in self.client.cookies.jar:
                    # 只提取夸克相关的重要Cookie
                    if cookie.domain and 'quark.cn' in cookie.domain:
                        cookies.append(f"{cookie.name}={cookie.value}")

                cookie_string = "; ".join(cookies)
                self.logger.debug(f"提取到Cookie: {len(cookies)}个")

                return cookie_string
            else:
                raise AuthenticationError("登录超时或失败")

        except Exception as e:
            if isinstance(e, AuthenticationError):
                raise
            raise AuthenticationError(f"API登录失败: {e}")


def api_login(timeout: int = 300) -> str:
    """
    API登录便捷函数

    Args:
        timeout: 超时时间（秒）

    Returns:
        Cookie字符串
    """
    login_manager = APILogin(timeout=timeout)
    return login_manager.login()



"""
交互式CLI模式
"""

import os
import shlex
from typing import List, Optional
from rich.console import Console
from rich.prompt import Prompt
from rich.table import Table
from rich.panel import Panel

from .utils import print_info, print_error, print_success, print_warning, get_client

from .commands.basic_fileops import upload_file
from .commands.share_commands import create_share, list_my_shares
from .commands.move_commands import move_files

console = Console()


class InteractiveShell:
    """交互式Shell"""

    def __init__(self):
        self.client = None
        self.current_folder_id = "0"
        self.current_folder_name = "根目录"
        self.running = True

        # 目录栈：存储 (folder_id, folder_name) 的路径
        self.directory_stack = [("0", "根目录")]

        # 命令映射
        self.commands = {
            'help': self.cmd_help,
            'h': self.cmd_help,
            '?': self.cmd_help,
            'exit': self.cmd_exit,
            'quit': self.cmd_exit,
            'q': self.cmd_exit,
            'ls': self.cmd_list,
            'list': self.cmd_list,
            'll': self.cmd_list_detailed,
            'cd': self.cmd_change_dir,
            'pwd': self.cmd_pwd,
            'search': self.cmd_search,
            'find': self.cmd_search,
            'download': self.cmd_download,
            'dl': self.cmd_download,
            'mkdir': self.cmd_mkdir,
            'rm': self.cmd_remove,
            'del': self.cmd_remove,
            'delete': self.cmd_remove,
            'rename': self.cmd_rename,
            'mv': self.cmd_rename,
            'info': self.cmd_info,
            'clear': self.cmd_clear,
            'cls': self.cmd_clear,
            'upload': self.cmd_upload,
            'up': self.cmd_upload,
            'share': self.cmd_share,
            'shares': self.cmd_shares,
            'move': self.cmd_move,
            'mv': self.cmd_move,
        }

    def start(self):
        """启动交互式模式"""
        console.print(Panel.fit(
            "[bold cyan]🌟 夸克网盘交互式CLI[/bold cyan]\n"
            "输入 'help' 查看可用命令\n"
            "输入 'exit' 退出程序",
            title="欢迎使用",
            border_style="cyan"
        ))

        # 检查登录状态
        try:
            self.client = get_client().__enter__()
            if not self.client.is_logged_in():
                print_error("未登录，请先使用 'quarkpan auth login' 登录")
                return

            print_success("✅ 已登录夸克网盘")
            print_info(f"当前位置: {self.current_folder_name}")

        except Exception as e:
            print_error(f"初始化失败: {e}")
            return

        # 主循环
        while self.running:
            try:
                # 显示提示符 - 使用友好显示名称
                display_name = self._get_display_name(self.current_folder_name)
                prompt = f"[cyan]quark[/cyan]:[blue]{display_name}[/blue]$ "
                command_line = Prompt.ask(prompt).strip()

                if not command_line:
                    continue

                # 解析命令
                try:
                    args = shlex.split(command_line)
                except ValueError as e:
                    print_error(f"命令解析错误: {e}")
                    continue

                if not args:
                    continue

                cmd = args[0].lower()
                cmd_args = args[1:]

                # 执行命令
                if cmd in self.commands:
                    try:
                        self.commands[cmd](cmd_args)
                    except KeyboardInterrupt:
                        print_info("\n命令被中断")
                    except Exception as e:
                        print_error(f"命令执行错误: {e}")
                else:
                    print_error(f"未知命令: {cmd}，输入 'help' 查看可用命令")

            except KeyboardInterrupt:
                print_info("\n使用 'exit' 退出程序")
            except EOFError:
                break

        # 清理
        try:
            if self.client:
                self.client.__exit__(None, None, None)
        except:
            pass

        print_info("再见！")

    def cmd_help(self, args: List[str]):
        """显示帮助信息"""
        table = Table(title="可用命令", show_header=True, header_style="bold magenta")
        table.add_column("命令", style="cyan", width=15)
        table.add_column("别名", style="dim", width=10)
        table.add_column("说明", style="white")

        commands_help = [
            ("help", "h, ?", "显示此帮助信息"),
            ("exit", "quit, q", "退出程序"),
            ("ls", "list", "列出当前目录文件"),
            ("ll", "", "详细列出当前目录文件"),
            ("cd <path>", "", "切换目录"),
            ("cd ..", "", "返回上级目录"),
            ("cd", "", "返回根目录"),
            ("pwd", "", "显示当前目录和路径"),
            ("search <keyword>", "find", "搜索文件"),
            ("download <path>", "dl", "下载文件"),
            ("mkdir <name>", "", "创建文件夹"),
            ("rm <path>...", "del", "删除文件/文件夹"),
            ("rename <old> <new>", "mv", "重命名文件/文件夹"),
            ("info <path>", "", "显示文件信息"),
            ("upload <file>", "up", "上传文件到当前目录"),
            ("share <path>", "", "创建分享链接"),
            ("shares", "", "查看我的分享列表"),
            ("move <src> <dst>", "mv", "移动文件到目标文件夹"),
            ("clear", "cls", "清屏"),
        ]

        for cmd, alias, desc in commands_help:
            table.add_row(cmd, alias, desc)

        console.print(table)

        console.print("\n[bold yellow]路径说明:[/bold yellow]")
        console.print("• 使用文件名: [cyan]文件.txt[/cyan]")
        console.print("• 使用相对路径: [cyan]文件夹/文件.txt[/cyan]")
        console.print("• 使用绝对路径: [cyan]/文件夹/文件.txt[/cyan]")
        console.print("• 文件夹路径末尾加/: [cyan]文件夹/[/cyan]")

    def cmd_exit(self, args: List[str]):
        """退出程序"""
        self.running = False

    def cmd_list(self, args: List[str]):
        """列出文件"""
        try:
            # 模拟调用list命令
            files = self.client.list_files(self.current_folder_id, size=50)  # type: ignore[attr-defined]
            file_list = files.get('data', {}).get('list', [])

            if not file_list:
                print_info("目录为空")
                return

            # 显示当前目录信息 - 使用友好显示名称
            display_name = self._get_display_name(self.current_folder_name, max_length=50)
            print_info(f"当前目录: {display_name}")
            print_info(f"共 {len(file_list)} 个项目\n")

            for i, file_info in enumerate(file_list, 1):
                name = file_info.get('file_name', '未知')
                file_type = file_info.get('file_type', 1)

                if file_type == 0:  # 文件夹
                    console.print(f"  {i:2d}. 📁 {name}/")
                else:  # 文件
                    size = file_info.get('size', 0)
                    size_str = self._format_size(size)
                    console.print(f"  {i:2d}. 📄 {name} [dim]({size_str})[/dim]")

        except Exception as e:
            print_error(f"列出文件失败: {e}")

    def cmd_list_detailed(self, args: List[str]):
        """详细列出文件"""
        try:
            files = self.client.list_files(self.current_folder_id, size=50)  # type: ignore[attr-defined]
            file_list = files.get('data', {}).get('list', [])

            if not file_list:
                print_info("目录为空")
                return

            # 使用友好显示名称作为表格标题
            display_name = self._get_display_name(self.current_folder_name, max_length=30)
            table = Table(title=f"目录内容: {display_name}")
            table.add_column("序号", style="dim", width=4)
            table.add_column("类型", style="cyan", width=4)
            table.add_column("名称", style="white")
            table.add_column("大小", style="green", width=10)

            for i, file_info in enumerate(file_list, 1):
                name = file_info.get('file_name', '未知')
                file_type = file_info.get('file_type', 1)
                size = file_info.get('size', 0)

                if file_type == 0:
                    table.add_row(str(i), "📁", f"{name}/", "-")
                else:
                    size_str = self._format_size(size)
                    table.add_row(str(i), "📄", name, size_str)

            console.print(table)

        except Exception as e:
            print_error(f"列出文件失败: {e}")

    def cmd_change_dir(self, args: List[str]):
        """切换目录"""
        if not args:
            # 回到根目录
            self._change_to_root()
            return

        path = args[0]

        try:
            if path == "..":
                # 返回上级目录
                self._change_to_parent()
                return

            file_id, file_type = self.client.resolve_path(path, self.current_folder_id)  # type: ignore[attr-defined]

            if file_type != 'folder':
                print_error(f"'{path}' 不是文件夹")
                return

            # 获取文件夹的真实名称（优先使用列表缓存中的名称）
            real_name = self.client.get_real_file_name(file_id)  # type: ignore[attr-defined]
            if real_name:
                folder_name = real_name
            else:
                # 如果缓存中没有，则使用API获取的名称
                folder_info = self.client.get_file_info(file_id)  # type: ignore[attr-defined]
                folder_name = folder_info.get('file_name', path)

            # 切换到新目录
            self._change_to_directory(file_id, folder_name)

        except Exception as e:
            print_error(f"切换目录失败: {e}")

    def cmd_pwd(self, args: List[str]):
        """显示当前目录"""
        display_name = self._get_display_name(self.current_folder_name, max_length=50)
        current_path = self._get_current_path()

        print_info(f"当前目录: {display_name}")
        print_info(f"完整路径: {current_path}")
        if len(self.current_folder_name) > 50:
            print_info(f"完整名称: {self.current_folder_name}")
        print_info(f"目录ID: {self.current_folder_id}")
        print_info(f"目录层级: {len(self.directory_stack) - 1}")

    def cmd_search(self, args: List[str]):
        """搜索文件"""
        if not args:
            print_error("请提供搜索关键词")
            return

        keyword = " ".join(args)
        print_info(f"搜索: {keyword}")

        try:
            # 简化的搜索实现
            results = self.client.search_files(keyword, size=20)  # type: ignore[attr-defined]
            file_list = results.get('data', {}).get('list', [])
            total = results.get('metadata', {}).get('_total', len(file_list))

            if not file_list:
                print_warning("没有找到匹配的文件")
                return

            print_success(f"找到 {total} 个结果（显示前20个）:")

            for i, file_info in enumerate(file_list, 1):
                name = file_info.get('file_name', '未知')
                file_type = file_info.get('file_type', 1)
                size = file_info.get('size', 0)

                if file_type == 0:
                    console.print(f"  {i:2d}. 📁 {name}/")
                else:
                    size_str = self._format_size(size)
                    console.print(f"  {i:2d}. 📄 {name} [dim]({size_str})[/dim]")

        except Exception as e:
            print_error(f"搜索失败: {e}")

    def cmd_download(self, args: List[str]):
        """下载文件"""
        if not args:
            print_error("请提供要下载的文件路径")
            return

        path = args[0]

        try:
            print_info(f"准备下载: {path}")

            def progress_callback(downloaded, total):
                if total > 0:
                    percent = (downloaded / total) * 100
                    print(f"\r下载进度: {percent:.1f}%", end="", flush=True)

            downloaded_path = self.client.download_file_by_name(  # type: ignore[attr-defined]
                path,
                current_folder_id=self.current_folder_id,
                progress_callback=progress_callback
            )

            print()  # 换行
            print_success(f"✅ 下载完成: {downloaded_path}")

        except Exception as e:
            print()  # 换行
            print_error(f"下载失败: {e}")

    def cmd_mkdir(self, args: List[str]):
        """创建文件夹"""
        if not args:
            print_error("请提供文件夹名称")
            return

        folder_name = args[0]

        try:
            result = self.client.create_folder(folder_name, self.current_folder_id)  # type: ignore[attr-defined]

            if result and result.get('status') == 200:
                print_success(f"✅ 文件夹创建成功: {folder_name}")
            else:
                error_msg = result.get('message', '未知错误')
                print_error(f"创建文件夹失败: {error_msg}")

        except Exception as e:
            print_error(f"创建文件夹失败: {e}")

    def cmd_remove(self, args: List[str]):
        """删除文件"""
        if not args:
            print_error("请提供要删除的文件路径")
            return

        try:
            print_warning(f"准备删除 {len(args)} 个文件/文件夹:")

            for i, path in enumerate(args, 1):
                print_info(f"  {i}. {path}")

            from rich.prompt import Confirm
            if not Confirm.ask("\n确定要删除这些文件/文件夹吗？"):
                print_info("取消删除操作")
                return

            result = self.client.delete_files_by_name(args, self.current_folder_id)  # type: ignore[attr-defined]

            if result and result.get('status') == 200:
                print_success(f"✅ 成功删除 {len(args)} 个文件/文件夹")
            else:
                error_msg = result.get('message', '未知错误')
                print_error(f"删除失败: {error_msg}")

        except Exception as e:
            print_error(f"删除失败: {e}")

    def cmd_rename(self, args: List[str]):
        """重命名文件"""
        if len(args) < 2:
            print_error("请提供原文件名和新文件名")
            return

        old_path = args[0]
        new_name = args[1]

        try:
            result = self.client.rename_file_by_name(  # type: ignore[attr-defined]
                old_path, new_name, self.current_folder_id)

            if result and result.get('status') == 200:
                print_success(f"✅ 重命名成功: {old_path} -> {new_name}")
            else:
                error_msg = result.get('message', '未知错误')
                print_error(f"重命名失败: {error_msg}")

        except Exception as e:
            print_error(f"重命名失败: {e}")

    def cmd_info(self, args: List[str]):
        """显示文件信息"""
        if not args:
            print_error("请提供文件路径")
            return

        path = args[0]

        try:
            file_info = self.client.get_file_info_by_name(path, self.current_folder_id)  # type: ignore[attr-defined]

            table = Table(title=f"文件信息: {path}")
            table.add_column("属性", style="cyan")
            table.add_column("值", style="white")

            table.add_row("文件名", file_info.get('file_name', '未知'))
            table.add_row("文件ID", file_info.get('fid', '未知'))
            table.add_row("类型", "文件夹" if file_info.get('file_type') == 0 else "文件")
            table.add_row("大小", self._format_size(file_info.get('size', 0)))
            table.add_row("格式", file_info.get('format_type', '未知'))

            console.print(table)

        except Exception as e:
            print_error(f"获取文件信息失败: {e}")

    def cmd_clear(self, args: List[str]):
        """清屏"""
        os.system('clear' if os.name == 'posix' else 'cls')

    def _format_size(self, size: int) -> str:
        """格式化文件大小"""
        if size < 1024:
            return f"{size} B"
        elif size < 1024 * 1024:
            return f"{size / 1024:.1f} KB"
        elif size < 1024 * 1024 * 1024:
            return f"{size / (1024 * 1024):.1f} MB"
        else:
            return f"{size / (1024 * 1024 * 1024):.1f} GB"

    def _get_display_name(self, folder_name: str, max_length: int = 20) -> str:
        """
        获取友好显示的文件夹名称

        Args:
            folder_name: 原始文件夹名称
            max_length: 最大显示长度

        Returns:
            友好显示的名称
        """
        if not folder_name or folder_name == "根目录":
            return "根目录"

        # 如果名称不太长，直接返回
        if len(folder_name) <= max_length:
            return folder_name

        # 对于长名称，进行智能截断
        # 优先保留开头和结尾的重要信息
        if len(folder_name) > max_length:
            # 计算截断位置
            start_len = max_length // 2 - 1
            end_len = max_length - start_len - 3  # 3个字符用于"..."

            if start_len > 0 and end_len > 0:
                return f"{folder_name[:start_len]}...{folder_name[-end_len:]}"
            else:
                # 如果太短，直接截断
                return f"{folder_name[:max_length-3]}..."

        return folder_name

    def _change_to_root(self):
        """切换到根目录"""
        self.current_folder_id = "0"
        self.current_folder_name = "根目录"
        self.directory_stack = [("0", "根目录")]
        print_info("已切换到根目录")

    def _change_to_parent(self):
        """返回上级目录"""
        if len(self.directory_stack) <= 1:
            print_warning("已经在根目录，无法返回上级目录")
            return

        # 弹出当前目录，返回上级
        self.directory_stack.pop()
        parent_id, parent_name = self.directory_stack[-1]

        self.current_folder_id = parent_id
        self.current_folder_name = parent_name

        display_name = self._get_display_name(parent_name)
        print_success(f"已返回上级目录: {display_name}")

    def _change_to_directory(self, folder_id: str, folder_name: str):
        """切换到指定目录"""
        # 添加到目录栈
        self.directory_stack.append((folder_id, folder_name))

        # 更新当前目录
        self.current_folder_id = folder_id
        self.current_folder_name = folder_name

        # 显示切换成功信息
        display_name = self._get_display_name(folder_name)
        print_success(f"已切换到: {display_name}")

    def _get_current_path(self) -> str:
        """获取当前路径字符串"""
        if len(self.directory_stack) <= 1:
            return "/"

        path_parts = []
        for _, name in self.directory_stack[1:]:  # 跳过根目录
            path_parts.append(name)

        return "/" + "/".join(path_parts)

    def cmd_upload(self, args: List[str]):
        """上传文件"""
        if not args:
            print_error("用法: upload <本地文件路径>")
            print_info("示例: upload /path/to/file.txt")
            return

        local_file_path = args[0]

        # 检查文件是否存在
        if not os.path.exists(local_file_path):
            print_error(f"文件不存在: {local_file_path}")
            return

        if not os.path.isfile(local_file_path):
            print_error(f"路径不是文件: {local_file_path}")
            return

        try:
            print_info(f"上传文件到当前目录: {self.current_folder_name}")

            # 调用上传函数，上传到当前目录
            upload_file(
                file_path=local_file_path,
                parent_folder_id=self.current_folder_id,
                folder_path=None,
                create_dirs=False
            )

        except Exception as e:
            print_error(f"上传失败: {e}")

    def cmd_share(self, args: List[str]):
        """创建分享链接"""
        if not args:
            print_error("用法: share <文件/文件夹路径> [选项]")
            print_info("示例: share 文件.txt")
            print_info("示例: share 文件夹/")
            print_info("选项:")
            print_info("  --title <标题>     设置分享标题")
            print_info("  --password <密码>  设置提取码")
            print_info("  --expire <天数>    设置过期天数(0=永久)")
            return

        file_path = args[0]

        # 解析选项
        title = ""
        password = None
        expire_days = 0

        i = 1
        while i < len(args):
            if args[i] == "--title" and i + 1 < len(args):
                title = args[i + 1]
                i += 2
            elif args[i] == "--password" and i + 1 < len(args):
                password = args[i + 1]
                i += 2
            elif args[i] == "--expire" and i + 1 < len(args):
                try:
                    expire_days = int(args[i + 1])
                except ValueError:
                    print_error("过期天数必须是数字")
                    return
                i += 2
            else:
                i += 1

        try:
            # 解析文件路径到文件ID
            file_id = self._resolve_path_to_id(file_path)
            if not file_id:
                print_error(f"无法找到文件: {file_path}")
                return

            print_info("创建分享链接...")

            # 调用分享函数
            create_share(
                file_paths=[file_id],
                title=title,
                expire_days=expire_days,
                password=password,
                use_id=True
            )

        except Exception as e:
            print_error(f"创建分享失败: {e}")

    def cmd_shares(self, args: List[str]):
        """查看我的分享列表"""
        # 解析参数
        page = 1
        size = 20

        i = 0
        while i < len(args):
            if args[i] == "--page" and i + 1 < len(args):
                try:
                    page = int(args[i + 1])
                except ValueError:
                    print_error("页码必须是数字")
                    return
                i += 2
            elif args[i] == "--size" and i + 1 < len(args):
                try:
                    size = int(args[i + 1])
                except ValueError:
                    print_error("每页数量必须是数字")
                    return
                i += 2
            else:
                i += 1

        try:
            print_info("获取分享列表...")

            # 调用分享列表函数
            list_my_shares(page=page, size=size)

        except Exception as e:
            print_error(f"获取分享列表失败: {e}")

    def cmd_move(self, args: List[str]):
        """移动文件"""
        if len(args) < 2:
            print_error("用法: move <源文件路径> <目标文件夹路径>")
            print_info("示例: move file.txt Documents/")
            print_info("示例: mv folder1/ folder2/")
            return

        source_path = args[0]
        target_path = args[1]

        try:
            # 解析源文件路径到文件ID
            source_file_id = self._resolve_path_to_id(source_path)
            if not source_file_id:
                print_error(f"无法找到源文件: {source_path}")
                return

            # 解析目标文件夹路径到文件夹ID
            target_folder_id = self._resolve_path_to_id(target_path)
            if not target_folder_id:
                print_error(f"无法找到目标文件夹: {target_path}")
                return

            # 检查目标是否为文件夹
            try:
                from ..services.name_resolver import NameResolver
                resolver = NameResolver(self.client.files)  # type: ignore[attr-defined]

                # 获取目标文件信息
                if target_path.startswith('/'):
                    _, target_info = resolver.resolve_path(target_path)
                else:
                    # 相对路径，从当前目录开始解析
                    if self.current_folder_id == "0":
                        full_path = f"/{target_path}"
                    else:
                        current_path = self._get_current_path()
                        full_path = f"{current_path}/{target_path}".replace("//", "/")
                    _, target_info = resolver.resolve_path(full_path)

                if not target_info.get('dir', False):  # type: ignore[attr-defined]
                    print_error(f"目标不是文件夹: {target_path}")
                    return

            except Exception as e:
                print_error(f"无法验证目标文件夹: {e}")
                return

            print_info(f"移动 '{source_path}' 到 '{target_path}'...")

            # 调用移动函数
            move_files(
                source_paths=[source_file_id],
                target_path=target_folder_id,
                use_id=True
            )

        except Exception as e:
            print_error(f"移动文件失败: {e}")

    def _resolve_path_to_id(self, path: str) -> Optional[str]:
        """解析路径到文件ID"""
        try:
            from ..services.name_resolver import NameResolver
            resolver = NameResolver(self.client.files)  # type: ignore[attr-defined]

            # 如果是绝对路径，从根目录开始解析
            if path.startswith('/'):
                file_id, _ = resolver.resolve_path(path)
            else:
                # 相对路径，从当前目录开始解析
                if self.current_folder_id == "0":
                    # 在根目录
                    file_id, _ = resolver.resolve_path(f"/{path}")
                else:
                    # 在子目录，需要构造完整路径
                    current_path = self._get_current_path()
                    full_path = f"{current_path}/{path}".replace("//", "/")
                    file_id, _ = resolver.resolve_path(full_path)

            return file_id
        except Exception:
            return None


def start_interactive():
    """启动交互式模式"""
    shell = InteractiveShell()
    shell.start()


if __name__ == "__main__":
    start_interactive()



# -*- coding: utf-8 -*-
"""
夸克网盘客户端主类
"""

from typing import Any, Callable, Dict, List, Optional, Tuple

from .auth import QuarkAuth
from .core.api_client import QuarkAPIClient
from .services.batch_share_service import BatchShareService
from .services.file_download_service import FileDownloadService
from .services.file_service import FileService
from .services.file_upload_service import FileUploadService
from .services.name_resolver import NameResolver
from .services.share_service import ShareService


class QuarkClient:
    """夸克网盘客户端主类"""

    def __init__(self, cookies: Optional[str] = None, auto_login: bool = True):
        """
        初始化夸克网盘客户端

        Args:
            cookies: Cookie字符串，如果为None则自动获取
            auto_login: 是否自动登录
        """
        # 初始化API客户端
        self.api_client = QuarkAPIClient(cookies=cookies, auto_login=auto_login)

        # 初始化服务
        self.files = FileService(self.api_client)
        self.upload = FileUploadService(self.api_client)
        self.download = FileDownloadService(self.api_client)
        self.shares = ShareService(self.api_client)
        self.batch_shares = BatchShareService(self.api_client)
        self.name_resolver = NameResolver(self.files)

        # 保存认证信息
        self._auth = None

    @property
    def auth(self) -> QuarkAuth:
        """获取认证管理器"""
        if not self._auth:
            self._auth = QuarkAuth()
        return self._auth

    def login(self, force_relogin: bool = False, use_qr: bool = True, method: str = "auto") -> str:
        """
        执行多层级登录

        Args:
            force_relogin: 是否强制重新登录
            use_qr: 是否使用二维码登录（兼容性参数）
            method: 登录方式 ("auto", "api", "simple")

        Returns:
            Cookie字符串
        """
        cookies = self.auth.login(force_relogin, use_qr, method)
        self.api_client.cookies = cookies
        return cookies

    def logout(self):
        """登出"""
        self.auth.logout()
        self.api_client.cookies = None

    def is_logged_in(self) -> bool:
        """检查是否已登录"""
        return self.auth.is_logged_in()

    # 文件管理快捷方法
    def list_files(self, folder_id: str = "0", **kwargs) -> Dict[str, Any]:
        """获取文件列表"""
        return self.files.list_files(folder_id, **kwargs)

    def get_file_info(self, file_id: str) -> Dict[str, Any]:
        """获取文件信息"""
        return self.files.get_file_info(file_id)

    def search_files(self, keyword: str, **kwargs) -> Dict[str, Any]:
        """搜索文件"""
        return self.files.search_files(keyword, **kwargs)

    def get_download_url(self, file_id: str) -> str:
        """获取下载链接"""
        return self.download.get_download_url(file_id)

    def get_download_urls(self, file_ids: List[str]) -> Dict[str, str]:
        """批量获取下载链接"""
        return self.download.get_download_urls(file_ids)

    def download_file(self, file_id: str, save_path: Optional[str] = None, **kwargs) -> str:
        """下载文件"""
        return self.download.download_file(file_id, save_path, **kwargs)

    def download_files(self, file_ids: List[str], save_dir: str = "downloads", **kwargs) -> List[str]:
        """批量下载文件"""
        return self.download.download_files(file_ids, save_dir, **kwargs)

    # 基于名称的操作方法
    def resolve_path(self, path: str, current_folder_id: str = "0") -> tuple:
        """解析文件路径到ID"""
        return self.name_resolver.resolve_path(path, current_folder_id)

    def delete_files_by_name(self, paths: List[str], current_folder_id: str = "0") -> Dict[str, Any]:
        """根据文件名删除文件"""
        resolved = self.name_resolver.resolve_multiple_paths(paths, current_folder_id)
        file_ids = [item[0] for item in resolved]
        return self.delete_files(file_ids)

    def rename_file_by_name(self, old_path: str, new_name: str, current_folder_id: str = "0") -> Dict[str, Any]:
        """根据文件名重命名文件"""
        file_id, _ = self.name_resolver.resolve_path(old_path, current_folder_id)
        return self.rename_file(file_id, new_name)

    def move_files_by_name(self, paths: List[str], target_path: str, current_folder_id: str = "0") -> Dict[str, Any]:
        """根据文件名移动文件"""
        # 解析源文件
        resolved = self.name_resolver.resolve_multiple_paths(paths, current_folder_id)
        file_ids = [item[0] for item in resolved]

        # 解析目标文件夹
        target_id, target_type = self.name_resolver.resolve_path(target_path, current_folder_id)
        if target_type != 'folder':
            raise ValueError(f"目标路径必须是文件夹: {target_path}")

        return self.move_files(file_ids, target_id)

    def download_file_by_name(
            self, path: str, save_path: Optional[str] = None, current_folder_id: str = "0", **kwargs) -> str:
        """根据文件名下载文件"""
        file_id, file_type = self.name_resolver.resolve_path(path, current_folder_id)
        if file_type != 'file':
            raise ValueError(f"只能下载文件，不能下载文件夹: {path}")
        return self.download_file(file_id, save_path, **kwargs)

    def get_file_info_by_name(self, path: str, current_folder_id: str = "0") -> Dict[str, Any]:
        """根据文件名获取文件信息"""
        file_id, _ = self.name_resolver.resolve_path(path, current_folder_id)
        return self.get_file_info(file_id)

    def get_real_file_name(self, file_id: str) -> Optional[str]:
        """获取文件的真实名称（从列表缓存中获取）"""
        return self.name_resolver.get_real_name(file_id)

    def list_files_with_details(self, **kwargs) -> Dict[str, Any]:
        """获取文件列表（增强版）"""
        return self.files.list_files_with_details(**kwargs)

    def search_files_advanced(self, keyword: str, **kwargs) -> Dict[str, Any]:
        """高级文件搜索"""
        return self.files.search_files_advanced(keyword, **kwargs)

    def create_folder(self, folder_name: str, parent_id: str = "0") -> Dict[str, Any]:
        """创建文件夹"""
        return self.files.create_folder(folder_name, parent_id)

    def delete_files(self, file_ids: List[str]) -> Dict[str, Any]:
        """删除文件"""
        return self.files.delete_files(file_ids)

    def rename_file(self, file_id: str, new_name: str) -> Dict[str, Any]:
        """重命名文件"""
        return self.files.rename_file(file_id, new_name)

    def batch_save_shares(
        self,
        share_urls: List[str],
        target_folder_id: str = "0",
        create_subfolder: bool = True
    ) -> List[Dict[str, Any]]:
        """
        批量转存分享链接

        Args:
            share_urls: 分享链接列表
            target_folder_id: 目标文件夹ID
            create_subfolder: 是否为每个分享创建子文件夹

        Returns:
            转存结果列表
        """
        results = []

        for i, share_url in enumerate(share_urls):
            try:
                # 如果需要创建子文件夹，生成文件夹名
                folder_name = None
                if create_subfolder:
                    folder_name = f"分享_{i+1}"

                result = self.save_shared_files(
                    share_url,
                    target_folder_id,
                    target_folder_name=folder_name
                )

                results.append({
                    'success': True,
                    'share_url': share_url,
                    'result': result
                })

            except Exception as e:
                results.append({
                    'success': False,
                    'share_url': share_url,
                    'error': str(e)
                })

        return results

    def sync_folder(
        self,
        local_path: str,
        remote_folder_id: str = "0",
        upload_new: bool = True,
        delete_remote: bool = False
    ) -> Dict[str, Any]:
        """
        同步本地文件夹到云端（占位符，需要实现上传功能）

        Args:
            local_path: 本地文件夹路径
            remote_folder_id: 远程文件夹ID
            upload_new: 是否上传新文件
            delete_remote: 是否删除远程多余文件

        Returns:
            同步结果
        """
        # TODO: 实现文件上传和同步逻辑
        _ = local_path, remote_folder_id, upload_new, delete_remote  # 参数将在未来实现中使用
        raise NotImplementedError("文件同步功能待实现")

    def get_storage_info(self) -> Dict[str, Any]:
        """
        获取存储空间信息

        Returns:
            存储信息
        """
        try:
            response = self.api_client.get('capacity')
            return response
        except Exception as e:
            return {'error': str(e)}

    def upload_file(
        self,
        file_path: str,
        parent_folder_id: str = "0",
        progress_callback: Optional[Callable] = None
    ) -> Dict[str, Any]:
        """
        上传文件到夸克网盘

        Args:
            file_path: 本地文件路径
            parent_folder_id: 父文件夹ID，默认为根目录
            progress_callback: 进度回调函数

        Returns:
            上传结果字典
        """
        return self.upload.upload_file(file_path, parent_folder_id, progress_callback)

    # 分享相关方法
    def create_share(
        self,
        file_ids: List[str],
        title: str = "",
        expire_days: int = 0,
        password: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        创建分享链接

        Args:
            file_ids: 文件ID列表
            title: 分享标题
            expire_days: 过期天数，0表示永久
            password: 提取码，None表示无密码

        Returns:
            分享信息，包含分享链接
        """
        return self.shares.create_share(file_ids, title, expire_days, password)

    def parse_share_url(self, share_url: str) -> Tuple[str, Optional[str]]:
        """
        解析分享链接

        Args:
            share_url: 分享链接

        Returns:
            (share_id, password) 元组
        """
        return self.shares.parse_share_url(share_url)

    def save_shared_files(
        self,
        share_url: str,
        target_folder_id: str = "0",
        target_folder_name: Optional[str] = None,
        file_filter: Optional[Callable] = None
    ) -> Dict[str, Any]:
        """
        转存分享的文件

        Args:
            share_url: 分享链接
            target_folder_id: 目标文件夹ID
            target_folder_name: 目标文件夹名称
            file_filter: 文件过滤函数

        Returns:
            转存结果
        """
        return self.shares.parse_and_save(share_url, target_folder_id, target_folder_name, file_filter)

    def get_my_shares(self, page: int = 1, size: int = 50) -> Dict[str, Any]:
        """
        获取我的分享列表

        Args:
            page: 页码
            size: 每页数量

        Returns:
            分享列表
        """
        return self.shares.get_my_shares(page, size)

    # 文件操作方法
    def move_files(
        self,
        file_ids: List[str],
        target_folder_id: str,
        exclude_fids: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """
        移动文件到指定文件夹

        Args:
            file_ids: 要移动的文件ID列表
            target_folder_id: 目标文件夹ID
            exclude_fids: 排除的文件ID列表

        Returns:
            移动结果
        """
        return self.files.move_files(file_ids, target_folder_id, exclude_fids)

    def close(self):
        """关闭客户端"""
        self.api_client.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        _ = exc_type, exc_val, exc_tb  # 参数未使用
        self.close()


# 便捷函数
def create_client(cookies: Optional[str] = None, auto_login: bool = True) -> QuarkClient:
    """创建夸克网盘客户端的便捷函数"""
    return QuarkClient(cookies=cookies, auto_login=auto_login)



# -*- coding: utf-8 -*-
"""
服务模块
"""

from .batch_share_service import BatchShareService
from .file_download_service import FileDownloadService
from .file_service import FileService
from .file_upload_service import FileUploadService
from .share_service import ShareService

__all__ = ['FileService', 'FileUploadService', 'FileDownloadService', 'ShareService', 'BatchShareService']



"""
夸克网盘登录模块
支持多种登录方式：API登录、简化登录、Playwright登录
"""

import json
import time
from typing import Dict, List, Optional

from ..config import get_config_dir
from ..exceptions import AuthenticationError, ConfigError
from ..utils.logger import get_logger


class QuarkAuth:
    """夸克网盘认证管理器"""

    def __init__(self, timeout: int = 300):
        """
        初始化认证管理器

        Args:
            timeout: 登录超时时间(秒)，默认5分钟
        """
        self.timeout = timeout
        self.config_dir = get_config_dir()
        self.cookies_file = self.config_dir / "cookies.json"
        self.logger = get_logger(__name__)

        # 确保配置目录存在
        self.config_dir.mkdir(parents=True, exist_ok=True)

    def _save_cookies(self, cookies: List[Dict]) -> None:
        """保存cookies到本地文件"""
        try:
            with open(self.cookies_file, 'w', encoding='utf-8') as f:
                json.dump({
                    'cookies': cookies,
                    'timestamp': int(time.time()),
                    'expires_at': self._get_cookies_expire_time(cookies)
                }, f, ensure_ascii=False, indent=2)
        except Exception as e:
            raise ConfigError(f"保存cookies失败: {e}")

    def _load_cookies(self) -> Optional[Dict]:
        """从本地文件加载cookies"""
        try:
            if not self.cookies_file.exists():
                return None

            with open(self.cookies_file, 'r', encoding='utf-8') as f:
                data = json.load(f)

            # 检查cookies是否过期
            if self._is_cookies_expired(data):
                return None

            return data
        except Exception:
            return None

    def _get_cookies_expire_time(self, cookies: List[Dict]) -> Optional[int]:
        """获取cookies的过期时间"""
        min_expire = None
        for cookie in cookies:
            # 检查所有cookie，不只是quark域名的
            if 'expires' in cookie and cookie['expires'] > 0:
                expire_time = cookie['expires']
                if min_expire is None or expire_time < min_expire:
                    min_expire = expire_time

        # 如果没有找到有效的过期时间，返回一个合理的默认值（7天后）
        if min_expire is None:
            import time
            min_expire = int(time.time()) + (7 * 24 * 3600)

        return min_expire

    def _is_cookies_expired(self, cookie_data: Dict) -> bool:
        """检查cookies是否过期"""
        import time
        current_time = int(time.time())

        # 如果过期时间无效（None、-1等），使用时间戳检查
        expires_at = cookie_data.get('expires_at')
        if expires_at is None or expires_at <= 0:
            # 如果没有过期时间信息，检查是否超过7天
            timestamp = cookie_data.get('timestamp', 0)
            age_seconds = current_time - timestamp
            return age_seconds > (7 * 24 * 3600)

        return current_time > expires_at

    def _cookies_to_dict(self, cookies: List[Dict]) -> Dict[str, str]:
        """将cookies列表转换为字典格式"""
        result = {}
        for cookie in cookies:
            # 包含所有cookie，不只是quark域名的
            result[cookie['name']] = cookie['value']
        return result

    def _cookies_to_string(self, cookies: List[Dict]) -> str:
        """将cookies列表转换为字符串格式"""
        cookie_dict = self._cookies_to_dict(cookies)
        return '; '.join([f"{key}={value}" for key, value in cookie_dict.items()])

    def login(self, force_relogin: bool = False, use_qr: bool = True, method: str = "auto") -> str:
        """
        执行多层级登录流程

        Args:
            force_relogin: 是否强制重新登录
            use_qr: 是否使用二维码登录（兼容性参数）
            method: 登录方式 ("auto", "api", "simple")

        Returns:
            Cookie字符串

        Raises:
            AuthenticationError: 登录失败
        """
        # 如果不是强制重新登录，先尝试使用已保存的cookies
        if not force_relogin:
            saved_cookies = self._load_cookies()
            if saved_cookies:
                self.logger.info("使用已保存的登录凭证")
                return self._cookies_to_string(saved_cookies['cookies'])

        # 根据指定方法进行登录
        if method == "auto":
            return self._auto_login()
        elif method == "api":
            return self._api_login()
        elif method == "simple":
            return self._simple_login()
        else:
            raise AuthenticationError(f"不支持的登录方式: {method}")

    def _auto_login(self) -> str:
        """
        自动选择最佳登录方式
        优先级: API登录 → 简化登录 → Playwright登录
        """
        self.logger.debug("开始自动登录流程")

        # 1. 尝试API登录
        try:
            self.logger.debug("尝试API登录...")
            return self._api_login()
        except Exception as e:
            self.logger.debug(f"API登录失败: {e}")

        # 2. 尝试简化登录
        try:
            self.logger.debug("尝试简化登录...")
            return self._simple_login()
        except Exception as e:
            self.logger.debug(f"简化登录失败: {e}")

        # 如果所有方式都失败了
        raise AuthenticationError("所有登录方式都失败了")

    def _api_login(self) -> str:
        """API登录方式"""
        try:
            from .api_login import APILogin

            api_login = APILogin(timeout=self.timeout)
            cookies = api_login.login()

            if cookies:
                # 解析cookies字符串为列表格式
                cookie_list = self._parse_cookie_string(cookies)

                # 保存cookies
                self._save_cookies(cookie_list)
                self.logger.debug("API登录成功")
                return cookies
            else:
                raise AuthenticationError("API登录返回空Cookie")

        except ImportError:
            raise AuthenticationError("API登录模块不可用")
        except Exception as e:
            raise AuthenticationError(f"API登录失败: {e}")

    def _simple_login(self) -> str:
        """简化登录方式"""
        try:
            from .simple_login import SimpleLogin

            simple_login = SimpleLogin()
            cookies = simple_login.login()

            if cookies:
                # 解析cookies字符串为列表格式
                cookie_list = self._parse_cookie_string(cookies)

                # 保存cookies
                self._save_cookies(cookie_list)
                self.logger.debug("简化登录成功")
                return cookies
            else:
                raise AuthenticationError("简化登录返回空Cookie")

        except ImportError:
            raise AuthenticationError("简化登录模块不可用")
        except Exception as e:
            raise AuthenticationError(f"简化登录失败: {e}")

    def _parse_cookie_string(self, cookie_string: str) -> List[Dict]:
        """将cookie字符串解析为列表格式"""
        cookies = []
        for pair in cookie_string.split('; '):
            if '=' in pair:
                name, value = pair.split('=', 1)
                cookies.append({
                    'name': name,
                    'value': value,
                    'domain': '.quark.cn',
                    'path': '/'
                })
        return cookies

    def _validate_cookies(self, cookies: List[Dict]) -> bool:
        """验证cookies是否有效"""
        # 检查是否包含夸克相关的cookies
        quark_cookies = [c for c in cookies if 'quark' in c.get('domain', '')]
        return len(quark_cookies) > 0

    def get_cookies(self, force_relogin: bool = False) -> str:
        """
        获取有效的cookies字符串

        Args:
            force_relogin: 是否强制重新登录

        Returns:
            Cookie字符串
        """
        # 如果不强制重新登录，先尝试返回已保存的cookies
        if not force_relogin:
            saved_cookies = self._load_cookies()
            if saved_cookies:
                cookies = saved_cookies['cookies']
                cookie_string = self._cookies_to_string(cookies)

                # 简单验证：检查是否有必要的cookie
                required_cookies = ['__pus', '__kps', '__uid']  # 夸克网盘的关键cookie
                has_required = all(required in cookie_string for required in required_cookies)

                if has_required:
                    self.logger.debug("使用已保存的有效Cookie")
                    return cookie_string
                else:
                    self.logger.warning("已保存的Cookie缺少必要字段，需要重新登录")

        # 如果没有有效的cookies或强制重新登录，则执行登录
        return self.login(force_relogin)

    def logout(self) -> None:
        """登出并清除本地cookies"""
        try:
            if self.cookies_file.exists():
                self.cookies_file.unlink()
            self.logger.debug("已清除登录信息")
        except Exception as e:
            self.logger.error(f"清除登录信息时出错: {e}")

    def is_logged_in(self) -> bool:
        """检查是否已登录"""
        saved_cookies = self._load_cookies()
        if saved_cookies is None:
            return False

        # 验证cookie是否有效
        try:
            cookies = saved_cookies['cookies']
            if not cookies:
                return False

            # 简单验证：检查是否有必要的cookie字段
            cookie_string = self._cookies_to_string(cookies)
            required_cookies = ['__pus', '__kps', '__uid']  # 夸克网盘的关键cookie

            for required in required_cookies:
                if required not in cookie_string:
                    return False

            return True

        except Exception:
            return False


# 便捷函数
def get_auth_cookies(timeout: int = 300, force_relogin: bool = False) -> str:
    """获取认证cookies的便捷函数"""
    auth = QuarkAuth(timeout=timeout)
    return auth.get_cookies(force_relogin=force_relogin)



"""
下载命令模块
"""

import os
from typing import List, Optional

import typer
from rich.console import Console
from rich.table import Table

from ..utils import (format_file_size, get_client, handle_api_error,
                     print_error, print_info, print_success, print_warning)

console = Console()
download_app = typer.Typer(help="📥 文件下载")


@download_app.command("file")
def download_file(
    file_id: str = typer.Argument(..., help="文件ID"),
    output_dir: str = typer.Option("downloads", "--output", "-o", help="下载目录"),
    filename: Optional[str] = typer.Option(None, "--name", "-n", help="自定义文件名")
):
    """下载单个文件"""
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            print_info(f"正在下载文件...")

            # 创建下载目录
            os.makedirs(output_dir, exist_ok=True)

            # 确定保存路径
            save_path = output_dir
            if filename:
                save_path = os.path.join(output_dir, filename)

            # 进度回调函数
            def progress_callback(downloaded, total):
                if total > 0:
                    percent = (downloaded / total) * 100
                    downloaded_mb = downloaded / (1024 * 1024)
                    total_mb = total / (1024 * 1024)
                    print(f"\r下载进度: {percent:.1f}% ({downloaded_mb:.1f}MB/{total_mb:.1f}MB)", end="", flush=True)
                else:
                    downloaded_mb = downloaded / (1024 * 1024)
                    print(f"\r已下载: {downloaded_mb:.1f}MB", end="", flush=True)

            # 下载文件
            downloaded_path = client.download_file(
                file_id,
                save_path,
                progress_callback=progress_callback
            )

            print()  # 换行
            print_success(f"✅ 文件下载成功: {downloaded_path}")

            # 显示文件信息
            if os.path.exists(downloaded_path):
                file_size = os.path.getsize(downloaded_path)
                print_info(f"文件大小: {format_file_size(file_size)}")

    except Exception as e:
        print()  # 换行
        handle_api_error(e, "下载文件")
        raise typer.Exit(1)


@download_app.command("files")
def download_files(
    file_ids: List[str] = typer.Argument(..., help="文件ID列表"),
    output_dir: str = typer.Option("downloads", "--output", "-o", help="下载目录")
):
    """批量下载文件"""
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            print_info(f"正在下载 {len(file_ids)} 个文件...")

            # 创建下载目录
            os.makedirs(output_dir, exist_ok=True)

            # 批量下载进度回调
            def batch_progress_callback(current_file, total_files, downloaded, total):
                if total > 0:
                    percent = (downloaded / total) * 100
                    downloaded_mb = downloaded / (1024 * 1024)
                    total_mb = total / (1024 * 1024)
                    print(
                        f"\r文件 {current_file}/{total_files}: {percent:.1f}% ({downloaded_mb:.1f}MB/{total_mb:.1f}MB)",
                        end="", flush=True)
                else:
                    downloaded_mb = downloaded / (1024 * 1024)
                    print(f"\r文件 {current_file}/{total_files}: {downloaded_mb:.1f}MB", end="", flush=True)

            # 批量下载文件
            downloaded_files = client.download_files(
                file_ids,
                output_dir,
                progress_callback=batch_progress_callback
            )

            print()  # 换行
            print_success(f"✅ 批量下载完成！成功下载 {len(downloaded_files)} 个文件")

            # 显示下载的文件列表
            if downloaded_files:
                table = Table(title="下载的文件")
                table.add_column("序号", style="dim", width=4)
                table.add_column("文件名", style="white")
                table.add_column("大小", style="green", width=12)

                for i, file_path in enumerate(downloaded_files, 1):
                    file_name = os.path.basename(file_path)
                    if os.path.exists(file_path):
                        file_size = os.path.getsize(file_path)
                        size_str = format_file_size(file_size)
                    else:
                        size_str = "未知"

                    table.add_row(str(i), file_name, size_str)

                console.print(table)

    except Exception as e:
        print()  # 换行
        handle_api_error(e, "批量下载文件")
        raise typer.Exit(1)


@download_app.command("folder")
def download_folder(
    folder_id: str = typer.Argument(..., help="文件夹ID"),
    output_dir: str = typer.Option("downloads", "--output", "-o", help="下载目录"),
    recursive: bool = typer.Option(True, "--recursive/--no-recursive", "-r", help="递归下载子文件夹")
):
    """下载文件夹"""
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            print_info(f"正在获取文件夹内容...")

            # 获取文件夹信息
            try:
                folder_info = client.get_file_info(folder_id)
                folder_name = folder_info.get('file_name', f'folder_{folder_id}')
            except:
                folder_name = f'folder_{folder_id}'

            print_info(f"文件夹: {folder_name}")

            # 创建下载目录
            download_path = os.path.join(output_dir, folder_name)
            os.makedirs(download_path, exist_ok=True)

            # 获取文件夹内容
            def download_folder_recursive(fid, path):
                files = client.list_files(fid, size=1000)  # 获取大量文件
                file_list = files.get('data', {}).get('list', [])

                downloaded_files = []

                for file_info in file_list:
                    file_id = file_info.get('fid', '')
                    file_name = file_info.get('file_name', '')
                    file_type = file_info.get('file_type', 1)

                    if file_type == 0:  # 文件夹
                        if recursive:
                            print_info(f"进入文件夹: {file_name}")
                            sub_path = os.path.join(path, file_name)
                            os.makedirs(sub_path, exist_ok=True)
                            sub_files = download_folder_recursive(file_id, sub_path)
                            downloaded_files.extend(sub_files)
                    else:  # 文件
                        try:
                            print_info(f"下载文件: {file_name}")

                            def progress_callback(downloaded, total):
                                if total > 0:
                                    percent = (downloaded / total) * 100
                                    print(f"\r  进度: {percent:.1f}%", end="", flush=True)

                            file_path = client.download_file(
                                file_id,
                                path,
                                progress_callback=progress_callback
                            )
                            print()  # 换行
                            downloaded_files.append(file_path)

                        except Exception as e:
                            print()
                            print_warning(f"下载文件 {file_name} 失败: {e}")

                return downloaded_files

            # 开始递归下载
            downloaded_files = download_folder_recursive(folder_id, download_path)

            print_success(f"✅ 文件夹下载完成！成功下载 {len(downloaded_files)} 个文件")
            print_info(f"下载位置: {download_path}")

    except Exception as e:
        handle_api_error(e, "下载文件夹")
        raise typer.Exit(1)


@download_app.command("info")
def show_download_info():
    """显示下载相关信息"""
    console.print("""
[bold cyan]📥 夸克网盘下载说明[/bold cyan]

[bold]下载文件:[/bold]
  quarkpan download file <file_id>     - 下载单个文件
  quarkpan download files <file_id>... - 批量下载文件
  quarkpan download folder <folder_id> - 下载文件夹

[bold]使用方法:[/bold]
  1. [bold green]直接下载到本地[/bold green]
  2. 支持单文件、批量文件、整个文件夹下载
  3. 自动创建下载目录和保持文件夹结构

[bold]示例:[/bold]
  # 下载单个文件
  quarkpan download file 0d51b7344d894d20a671a5c567383749

  # 下载到指定目录
  quarkpan download file 0d51b7344d894d20a671a5c567383749 -o /path/to/downloads

  # 批量下载文件
  quarkpan download files file_id1 file_id2 file_id3

  # 下载整个文件夹
  quarkpan download folder folder_id

[bold yellow]注意事项:[/bold yellow]
  • 需要先登录夸克网盘账号
  • 下载速度取决于网络和夸克网盘限制
  • 大文件下载会显示进度条
  • 下载失败的文件会跳过并继续下载其他文件
  • 文件夹下载会保持原有的目录结构

[bold]功能特点:[/bold]
  • ✅ 直接下载，无需浏览器
  • ✅ 支持进度显示
  • ✅ 支持批量下载
  • ✅ 支持文件夹递归下载
  • ✅ 自动重试机制
""")


if __name__ == "__main__":
    download_app()



"""
基础文件操作命令
"""

from typing import List, Optional

import typer
from rich.prompt import Confirm

from ..utils import (get_client, handle_api_error, print_error, print_info,
                     print_success, print_warning)


def create_folder(folder_name: str, parent_id: str = "0"):
    """创建文件夹"""
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            print_info(f"正在创建文件夹: {folder_name}")

            result = client.create_folder(folder_name, parent_id)

            if result and result.get('status') == 200:
                print_success(f"✅ 文件夹创建成功: {folder_name}")

                # 显示创建的文件夹信息
                if 'data' in result:
                    folder_info = result['data']
                    folder_id = folder_info.get('fid', '')
                    if folder_id:
                        print_info(f"文件夹ID: {folder_id}")
            else:
                error_msg = result.get('message', '未知错误')
                print_error(f"创建文件夹失败: {error_msg}")
                raise typer.Exit(1)

    except Exception as e:
        handle_api_error(e, "创建文件夹")
        raise typer.Exit(1)


def delete_files(paths: List[str], force: bool = False, use_id: bool = False):
    """删除文件或文件夹"""
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            # 解析路径或使用ID
            if use_id:
                file_ids = paths
                # 显示要删除的文件信息
                print_warning(f"准备删除 {len(file_ids)} 个文件/文件夹:")

                for i, file_id in enumerate(file_ids, 1):
                    try:
                        file_info = client.get_file_info(file_id)
                        file_name = file_info.get('file_name', file_id)
                        file_type = "文件夹" if file_info.get('file_type') == 0 else "文件"
                        print_info(f"  {i}. {file_type}: {file_name}")
                    except:
                        print_info(f"  {i}. ID: {file_id}")
            else:
                # 使用路径解析
                print_warning(f"准备删除 {len(paths)} 个文件/文件夹:")

                resolved_items = []
                for i, path in enumerate(paths, 1):
                    try:
                        file_id, file_type = client.resolve_path(path)
                        file_info = client.get_file_info(file_id)
                        file_name = file_info.get('file_name', path)
                        type_name = "文件夹" if file_type == 'folder' else "文件"
                        print_info(f"  {i}. {type_name}: {file_name} (路径: {path})")
                        resolved_items.append(file_id)
                    except Exception as e:
                        print_error(f"  {i}. 无法解析路径 '{path}': {e}")
                        raise typer.Exit(1)

                file_ids = resolved_items

            # 确认删除
            if not force:
                if not Confirm.ask("\n确定要删除这些文件/文件夹吗？"):
                    print_info("取消删除操作")
                    return

            print_info("正在删除文件...")

            if use_id:
                result = client.delete_files(file_ids)
            else:
                result = client.delete_files_by_name(paths)

            if result and result.get('status') == 200:
                print_success(f"✅ 成功删除 {len(file_ids)} 个文件/文件夹")
            else:
                error_msg = result.get('message', '未知错误')
                print_error(f"删除失败: {error_msg}")
                raise typer.Exit(1)

    except Exception as e:
        handle_api_error(e, "删除文件")
        raise typer.Exit(1)


def rename_file(path: str, new_name: str, use_id: bool = False):
    """重命名文件或文件夹"""
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            # 解析路径或使用ID
            if use_id:
                file_id = path
                try:
                    file_info = client.get_file_info(file_id)
                    old_name = file_info.get('file_name', file_id)
                    file_type = "文件夹" if file_info.get('file_type') == 0 else "文件"
                    print_info(f"当前{file_type}名称: {old_name}")
                    print_info(f"新{file_type}名称: {new_name}")
                except:
                    print_info(f"文件ID: {file_id}")
                    print_info(f"新名称: {new_name}")

                result = client.rename_file(file_id, new_name)
            else:
                try:
                    file_id, file_type = client.resolve_path(path)
                    file_info = client.get_file_info(file_id)
                    old_name = file_info.get('file_name', path)
                    type_name = "文件夹" if file_type == 'folder' else "文件"
                    print_info(f"当前{type_name}名称: {old_name} (路径: {path})")
                    print_info(f"新{type_name}名称: {new_name}")
                except Exception as e:
                    print_error(f"无法解析路径 '{path}': {e}")
                    raise typer.Exit(1)

                result = client.rename_file_by_name(path, new_name)

            print_info("正在重命名...")

            if result and result.get('status') == 200:
                print_success(f"✅ 重命名成功: {new_name}")
            else:
                error_msg = result.get('message', '未知错误')
                print_error(f"重命名失败: {error_msg}")
                raise typer.Exit(1)

    except Exception as e:
        handle_api_error(e, "重命名文件")
        raise typer.Exit(1)


def file_info(file_id: str):
    """获取文件详细信息"""
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            print_info(f"正在获取文件信息: {file_id}")

            file_info = client.get_file_info(file_id)

            if file_info:
                from rich.console import Console
                from rich.table import Table

                console = Console()
                table = Table(title=f"文件信息")
                table.add_column("属性", style="cyan")
                table.add_column("值", style="white")

                table.add_row("文件名", file_info.get('file_name', '未知'))
                table.add_row("文件ID", file_info.get('fid', '未知'))
                table.add_row("类型", "文件夹" if file_info.get('file_type') == 0 else "文件")
                table.add_row("大小", _format_size(file_info.get('size', 0)))
                table.add_row("格式", file_info.get('format_type', '未知'))
                table.add_row("创建时间", file_info.get('created_at', '未知'))
                table.add_row("修改时间", file_info.get('updated_at', '未知'))

                console.print(table)
            else:
                print_error("无法获取文件信息")
                raise typer.Exit(1)

    except Exception as e:
        handle_api_error(e, "获取文件信息")
        raise typer.Exit(1)


def browse_folder(folder_id: str = "0"):
    """交互式文件夹浏览"""
    # TODO: 实现交互式浏览功能
    _ = folder_id  # 参数将在未来实现中使用
    print_warning("交互式浏览功能正在开发中...")
    print_info("请使用 'quarkpan interactive' 启动完整的交互式模式")


def goto_folder(target: str, current_folder: str = "0"):
    """智能进入文件夹"""
    # TODO: 实现智能导航功能
    _ = target, current_folder  # 参数将在未来实现中使用
    print_warning("智能导航功能正在开发中...")
    print_info("请使用 'quarkpan interactive' 启动完整的交互式模式")


def get_download_link(file_id: str):
    """获取文件下载链接"""
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            print_info(f"正在获取下载链接: {file_id}")

            download_url = client.get_download_url(file_id)

            if download_url:
                print_success("下载链接获取成功:")
                print_info(download_url)
            else:
                print_error("无法获取下载链接")
                raise typer.Exit(1)

    except Exception as e:
        handle_api_error(e, "获取下载链接")
        raise typer.Exit(1)


def _format_size(size: int) -> str:
    """格式化文件大小"""
    if size < 1024:
        return f"{size} B"
    elif size < 1024 * 1024:
        return f"{size / 1024:.1f} KB"
    elif size < 1024 * 1024 * 1024:
        return f"{size / (1024 * 1024):.1f} MB"
    else:
        return f"{size / (1024 * 1024 * 1024):.1f} GB"


def _resolve_folder_path(client, folder_path: str, create_dirs: bool = False) -> str:
    """
    解析文件夹路径到文件夹ID

    Args:
        client: QuarkClient实例
        folder_path: 文件夹路径，如 '/Documents/Photos'
        create_dirs: 是否自动创建不存在的文件夹

    Returns:
        文件夹ID
    """
    from ...services.name_resolver import NameResolver

    resolver = NameResolver(client.files)

    try:
        # 尝试解析路径
        folder_id, file_type = resolver.resolve_path(folder_path)

        if file_type != 'folder':
            raise ValueError(f"路径 '{folder_path}' 不是文件夹")

        return folder_id

    except Exception:
        if not create_dirs:
            raise ValueError(f"文件夹路径不存在: {folder_path}。使用 --create-dirs 自动创建")

        # 自动创建文件夹路径
        return _create_folder_path(client, folder_path)


def _create_folder_path(client, folder_path: str) -> str:
    """
    递归创建文件夹路径

    Args:
        client: QuarkClient实例
        folder_path: 文件夹路径

    Returns:
        最终文件夹ID
    """
    # 处理绝对路径
    if folder_path.startswith('/'):
        current_folder_id = "0"  # 根目录
        folder_path = folder_path[1:]  # 移除开头的/
    else:
        current_folder_id = "0"  # 默认从根目录开始

    # 如果路径为空，返回根目录
    if not folder_path:
        return current_folder_id

    # 分割路径
    parts = [p for p in folder_path.split('/') if p]

    from ...services.name_resolver import NameResolver
    resolver = NameResolver(client.files)

    # 逐级创建文件夹
    for part in parts:
        try:
            # 尝试查找现有文件夹
            current_folder_id = resolver._find_in_folder(part, current_folder_id, 'folder')
        except:
            # 文件夹不存在，创建它
            print_info(f"创建文件夹: {part}")
            result = client.create_folder(part, current_folder_id)

            if result and result.get('status') == 200:
                folder_info = result.get('data', {})
                current_folder_id = folder_info.get('fid', '')
                if not current_folder_id:
                    raise ValueError(f"创建文件夹失败: {part}")
            else:
                error_msg = result.get('message', '未知错误')
                raise ValueError(f"创建文件夹失败: {part} - {error_msg}")

    return current_folder_id


def upload_file(file_path: str, parent_folder_id: str = "0", folder_path: Optional[str] = None, create_dirs: bool = False):
    """上传文件到夸克网盘"""
    from pathlib import Path

    from rich.console import Console
    from rich.progress import (BarColumn, Progress, SpinnerColumn,
                               TaskProgressColumn, TextColumn,
                               TimeRemainingColumn)

    console = Console()

    try:
        file_path_obj = Path(file_path)
        if not file_path_obj.exists():
            print_error(f"文件不存在: {file_path}")
            raise typer.Exit(1)

        if not file_path_obj.is_file():
            print_error(f"路径不是文件: {file_path}")
            raise typer.Exit(1)

        file_size = file_path_obj.stat().st_size
        print_info(f"上传 {file_path_obj.name} ({_format_size(file_size)})")

        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            # 处理文件夹路径参数
            if folder_path:
                try:
                    parent_folder_id = _resolve_folder_path(client, folder_path, create_dirs)
                    print_info(f"目标文件夹: {folder_path}")
                except Exception as e:
                    print_error(f"解析文件夹路径失败: {e}")
                    raise typer.Exit(1)

            # 创建进度条
            with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                BarColumn(),
                TaskProgressColumn(),
                TimeRemainingColumn(),
                console=console
            ) as progress:

                task = progress.add_task("上传中...", total=100)

                def progress_callback(percent: int, message: str):
                    progress.update(task, completed=percent, description=message)

                # 开始上传
                result = client.upload_file(
                    file_path=str(file_path),
                    parent_folder_id=parent_folder_id,
                    progress_callback=progress_callback
                )

            # 上传完成后，在进度条外显示结果
            if result.get('status') == 'success':
                md5_info = f" MD5: {result.get('md5', 'N/A')}" if console.is_terminal else ""
                print_success(f"上传成功{md5_info}")
            else:
                print_error("上传失败")
                raise typer.Exit(1)

    except Exception as e:
        handle_api_error(e, "上传文件")
        raise typer.Exit(1)



# -*- coding: utf-8 -*-
"""
文件管理服务
"""

from typing import Any, Dict, List, Optional

from ..core.api_client import QuarkAPIClient
from ..exceptions import APIError, FileNotFoundError


class FileService:
    """文件管理服务"""

    def __init__(self, client: QuarkAPIClient):
        """
        初始化文件服务

        Args:
            client: API客户端实例
        """
        self.client = client

    def list_files(
        self,
        folder_id: str = "0",
        page: int = 1,
        size: int = 50,
        sort_field: str = "file_name",
        sort_order: str = "asc"
    ) -> Dict[str, Any]:
        """
        获取文件列表

        Args:
            folder_id: 文件夹ID，"0"表示根目录
            page: 页码，从1开始
            size: 每页数量
            sort_field: 排序字段 (file_name, file_size, updated_at等)
            sort_order: 排序方向 (asc, desc)

        Returns:
            包含文件列表的字典
        """
        params = {
            'pdir_fid': folder_id,
            '_page': page,
            '_size': size,
            '_sort': f"{sort_field}:{sort_order}"
        }

        try:
            response = self.client.get('file/sort', params=params)
            return response
        except APIError as e:
            if 'not found' in str(e).lower():
                raise FileNotFoundError(f"文件夹不存在: {folder_id}")
            raise

    def get_file_info(self, file_id: str) -> Dict[str, Any]:
        """
        获取文件详细信息

        Args:
            file_id: 文件ID

        Returns:
            文件信息字典
        """
        if not file_id or file_id == "0":
            raise ValueError("无效的文件ID")

        params = {'fids': file_id}

        try:
            response = self.client.get('file', params=params)

            # 检查响应格式
            if isinstance(response, dict) and 'data' in response:
                data = response['data']
                if isinstance(data, dict) and 'list' in data:
                    file_list = data['list']
                    if file_list and len(file_list) > 0:
                        # 查找匹配的文件ID
                        for file_info in file_list:
                            if file_info.get('fid') == file_id:
                                return file_info

                        # 如果没有找到精确匹配，返回第一个
                        return file_list[0]
                elif isinstance(data, list) and len(data) > 0:
                    # 兼容旧格式
                    return data[0]

            raise FileNotFoundError(f"文件不存在: {file_id}")

        except APIError as e:
            if 'not found' in str(e).lower():
                raise FileNotFoundError(f"文件不存在: {file_id}")
            raise

    def create_folder(self, folder_name: str, parent_id: str = "0") -> Dict[str, Any]:
        """
        创建文件夹

        Args:
            folder_name: 文件夹名称
            parent_id: 父文件夹ID，"0"表示根目录

        Returns:
            创建结果
        """
        data = {
            'pdir_fid': parent_id,
            'file_name': folder_name,
            'dir_init_lock': False,
            'dir_path': ''
        }

        # 添加查询参数
        params = {
            'pr': 'ucpro',
            'fr': 'pc',
            'uc_param_str': ''
        }

        response = self.client.post('file', json_data=data, params=params)
        return response

    def delete_files(self, file_ids: List[str]) -> Dict[str, Any]:
        """
        删除文件/文件夹

        Args:
            file_ids: 文件ID列表

        Returns:
            删除结果
        """
        data = {
            'action_type': 2,  # 删除操作
            'filelist': file_ids,
            'exclude_fids': []
        }

        # 添加查询参数
        params = {
            'pr': 'ucpro',
            'fr': 'pc',
            'uc_param_str': ''
        }

        response = self.client.post('file/delete', json_data=data, params=params)
        return response

    def rename_file(self, file_id: str, new_name: str) -> Dict[str, Any]:
        """
        重命名文件/文件夹

        Args:
            file_id: 文件ID
            new_name: 新名称

        Returns:
            重命名结果
        """
        data = {
            'fid': file_id,
            'file_name': new_name
        }

        # 添加查询参数
        params = {
            'pr': 'ucpro',
            'fr': 'pc',
            'uc_param_str': ''
        }

        response = self.client.post('file/rename', json_data=data, params=params)
        return response

    def search_files(
        self,
        keyword: str,
        folder_id: str = "0",
        page: int = 1,
        size: int = 50,
        sort_field: str = "file_type",
        sort_order: str = "desc"
    ) -> Dict[str, Any]:
        """
        搜索文件

        Args:
            keyword: 搜索关键词
            folder_id: 搜索范围文件夹ID，"0"表示全盘搜索（暂不支持）
            page: 页码
            size: 每页数量
            sort_field: 排序字段
            sort_order: 排序方向

        Returns:
            搜索结果
        """
        params = {
            'q': keyword,
            '_page': page,
            '_size': size,
            '_fetch_total': 1,
            '_sort': f'{sort_field}:{sort_order},updated_at:desc',
            '_is_hl': 1  # 启用高亮
        }

        # 注意：夸克网盘的搜索API似乎不支持文件夹范围限制
        _ = folder_id  # folder_id参数暂时不使用

        response = self.client.get('file/search', params=params)
        return response

    def get_folder_tree(self, folder_id: str = "0", max_depth: int = 3) -> Dict[str, Any]:
        """
        获取文件夹树结构

        Args:
            folder_id: 根文件夹ID
            max_depth: 最大深度

        Returns:
            文件夹树结构
        """
        params = {
            'pdir_fid': folder_id,
            'max_depth': max_depth
        }

        response = self.client.get('file/tree', params=params)
        return response

    def get_storage_info(self) -> Dict[str, Any]:
        """
        获取存储空间信息

        Returns:
            存储空间信息
        """
        response = self.client.get('capacity')
        return response

    def list_files_with_details(
        self,
        folder_id: str = "0",
        page: int = 1,
        size: int = 50,
        sort_field: str = "file_name",
        sort_order: str = "asc",
        include_folders: bool = True,
        include_files: bool = True
    ) -> Dict[str, Any]:
        """
        获取文件列表（增强版，支持过滤）

        Args:
            folder_id: 文件夹ID，"0"表示根目录
            page: 页码，从1开始
            size: 每页数量
            sort_field: 排序字段
            sort_order: 排序方向
            include_folders: 是否包含文件夹
            include_files: 是否包含文件

        Returns:
            包含文件列表的字典
        """
        response = self.list_files(folder_id, page, size, sort_field, sort_order)

        # 如果需要过滤，则处理响应数据
        if not include_folders or not include_files:
            if isinstance(response, dict) and 'data' in response:
                file_list = response['data'].get('list', [])
                filtered_list = []

                for file_info in file_list:
                    file_type = file_info.get('file_type', 0)
                    is_folder = file_type == 0

                    if (is_folder and include_folders) or (not is_folder and include_files):
                        filtered_list.append(file_info)

                response['data']['list'] = filtered_list
                response['data']['filtered_total'] = len(filtered_list)

        return response

    def search_files_advanced(
        self,
        keyword: str,
        folder_id: str = "0",
        page: int = 1,
        size: int = 50,
        file_extensions: Optional[List[str]] = None,
        min_size: Optional[int] = None,
        max_size: Optional[int] = None,
        sort_field: str = "file_type",
        sort_order: str = "desc"
    ) -> Dict[str, Any]:
        """
        高级文件搜索（客户端过滤）

        Args:
            keyword: 搜索关键词
            folder_id: 搜索范围文件夹ID（暂不支持）
            page: 页码
            size: 每页数量
            file_extensions: 文件扩展名过滤 (如: ['pdf', 'doc', 'txt'])
            min_size: 最小文件大小（字节）
            max_size: 最大文件大小（字节）
            sort_field: 排序字段
            sort_order: 排序方向

        Returns:
            搜索结果
        """
        # 如果没有过滤条件，直接返回基础搜索结果
        if not file_extensions and min_size is None and max_size is None:
            return self.search_files(keyword, folder_id, page, size, sort_field, sort_order)

        # 获取更多结果用于客户端过滤
        search_size = max(size * 3, 100)
        response = self.search_files(keyword, folder_id, 1, search_size, sort_field, sort_order)

        # 应用客户端过滤器
        if isinstance(response, dict) and 'data' in response:
            file_list = response['data'].get('list', [])
            filtered_list = []

            for file_info in file_list:
                # 文件扩展名过滤
                if file_extensions:
                    file_name = file_info.get('file_name', '').lower()
                    file_ext = file_name.split('.')[-1] if '.' in file_name else ''
                    if file_ext not in [ext.lower() for ext in file_extensions]:
                        continue

                # 文件大小过滤
                file_size = file_info.get('size', 0)
                if min_size is not None and file_size < min_size:
                    continue
                if max_size is not None and file_size > max_size:
                    continue

                filtered_list.append(file_info)

            # 应用分页到过滤后的结果
            start_idx = (page - 1) * size
            end_idx = start_idx + size
            paginated_list = filtered_list[start_idx:end_idx]

            response['data']['list'] = paginated_list
            response['data']['filtered_total'] = len(filtered_list)
            # 更新metadata中的总数
            if 'metadata' in response:
                response['metadata']['_total'] = len(filtered_list)
                response['metadata']['_count'] = len(paginated_list)

        return response

    def get_file_path(self, file_id: str) -> str:
        """
        获取文件的完整路径

        Args:
            file_id: 文件ID

        Returns:
            文件路径字符串
        """
        try:
            file_info = self.get_file_info(file_id)
            # 这里需要根据实际API响应结构来获取路径
            # 可能需要递归获取父文件夹信息来构建完整路径
            return file_info.get('file_path', file_info.get('file_name', ''))
        except Exception:
            return ""

    def move_files(
        self,
        file_ids: List[str],
        target_folder_id: str,
        exclude_fids: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """
        移动文件到指定文件夹

        Args:
            file_ids: 要移动的文件ID列表
            target_folder_id: 目标文件夹ID
            exclude_fids: 排除的文件ID列表

        Returns:
            移动结果
        """
        data = {
            'action_type': 1,  # 移动操作
            'to_pdir_fid': target_folder_id,
            'filelist': file_ids,
            'exclude_fids': exclude_fids or []
        }

        response = self.client.post('file/move', json_data=data)

        if not response.get('status') == 200:
            raise APIError(f"移动文件失败: {response.get('message', '未知错误')}")

        data = response.get('data', {})
        task_id = data.get('task_id')
        finish = data.get('finish', False)

        if finish:
            # 同步完成，直接返回结果
            return response
        elif task_id:
            # 异步任务，需要轮询状态
            return self._wait_for_move_task(task_id, response.get('metadata', {}).get('tq_gap', 500))
        else:
            raise APIError("移动任务创建失败，无法获取任务ID")

    def _wait_for_move_task(self, task_id: str, poll_interval: int = 500) -> Dict[str, Any]:
        """
        等待移动任务完成

        Args:
            task_id: 任务ID
            poll_interval: 轮询间隔（毫秒）

        Returns:
            任务完成结果
        """
        import time

        max_retries = 30  # 最多等待15秒
        retry_count = 0

        while retry_count < max_retries:
            try:
                task_response = self.client.get(
                    'task',
                    params={
                        'task_id': task_id,
                        'retry_index': retry_count
                    }
                )

                if task_response.get('status') == 200:
                    task_data = task_response.get('data', {})

                    # 检查任务状态
                    if task_data.get('status') == 2:  # 任务完成
                        return task_response
                    elif task_data.get('status') == 3:  # 任务失败
                        raise APIError(f"移动任务失败: {task_data.get('message', '任务失败')}")

                retry_count += 1
                time.sleep(poll_interval / 1000)  # 转换为秒

            except Exception as e:
                if retry_count >= max_retries - 1:
                    raise APIError(f"移动任务轮询失败: {e}")
                retry_count += 1
                time.sleep(poll_interval / 1000)

        raise APIError("移动任务超时")



"""
配置管理模块
"""

import os
from pathlib import Path
from typing import Any, Dict


def get_config_dir() -> Path:
    """获取配置目录路径"""
    # 优先使用环境变量
    config_dir = os.getenv('QUARK_CONFIG_DIR')
    if config_dir:
        return Path(config_dir)

    # 默认使用当前目录下的config文件夹
    return Path.cwd() / 'config'


def get_default_headers() -> Dict[str, str]:
    """获取默认的HTTP请求头"""
    return {
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko)'
                      ' Chrome/94.0.4606.71 Safari/537.36 Core/1.94.225.400 QQBrowser/12.2.5544.400',
        'origin': 'https://pan.quark.cn',
        'referer': 'https://pan.quark.cn/',
        'accept-language': 'zh-CN,zh;q=0.9',
        'accept': 'application/json, text/plain, */*',
        'content-type': 'application/json',
    }


class Config:
    """配置类"""

    # API相关配置
    BASE_URL = 'https://drive-pc.quark.cn/1/clouddrive'
    SHARE_BASE_URL = 'https://drive.quark.cn/1/clouddrive'
    ACCOUNT_URL = 'https://pan.quark.cn/account'

    # 默认参数
    DEFAULT_PARAMS = {
        'pr': 'ucpro',
        'fr': 'pc',
        'uc_param_str': '',
    }

    # 请求超时设置
    REQUEST_TIMEOUT = 60.0

    # 重试设置
    MAX_RETRIES = 3
    RETRY_DELAY = 1.0

    # 分页设置
    DEFAULT_PAGE_SIZE = 50
    MAX_PAGE_SIZE = 100

    # 文件下载设置
    DOWNLOAD_CHUNK_SIZE = 8192
    DOWNLOAD_DIR = 'downloads'



"""
认证模块
"""

from .login import QuarkAuth, get_auth_cookies

# 导入具体的登录实现
try:
    from .api_login import APILogin, api_login
    _api_available = True
except ImportError:
    _api_available = False

try:
    from .simple_login import SimpleLogin, simple_login
    _simple_available = True
except ImportError:
    _simple_available = False

# 构建导出列表
__all__ = ['QuarkAuth', 'get_auth_cookies']

if _api_available:
    __all__.extend(['APILogin', 'api_login'])

if _simple_available:
    __all__.extend(['SimpleLogin', 'simple_login'])



#!/usr/bin/env python3
from pathlib import Path

from .logger import get_logger


def print_ascii_qr(text: str):
    logger = get_logger(__name__)
    try:
        import qrcode
        qr = qrcode.QRCode(border=1)  # 边框小一些，方便在终端放大显示
        qr.add_data(text)
        qr.make(fit=True)
        # 反相打印，提升终端显示对比度（部分终端/主题需要关闭 invert）
        qr.print_ascii(invert=True)
    except Exception as e:
        logger.warning(f"ASCII QR render failed: {e}")


def display_qr_code(qr_image_path: str):
    """
    显示二维码到终端

    由于QR码现在是从URL直接生成的，我们可以通过读取图片元数据或
    直接使用生成时的URL来显示ASCII二维码，而不需要复杂的图像解码
    """
    logger = get_logger(__name__)
    qr_path = Path(qr_image_path)

    if not qr_path.exists():
        logger.error(f"二维码文件不存在: {qr_image_path}")
        return

    # 由于我们知道这是从URL生成的，但没有直接方式获取原始URL，
    # 我们可以尝试从文件名或配置中获取，或者简单提示用户
    logger.info("二维码已生成，请使用夸克APP扫描")
    print(f"二维码文件位置: {qr_image_path}")


def display_qr_from_url(url: str):
    """
    直接从URL生成并显示ASCII二维码到终端
    这是更直接的方式，不需要先生成图片文件
    """
    print_ascii_qr(url)



"""
移动文件相关命令
"""

from typing import List

import typer

from ..utils import (get_client, handle_api_error, print_error, print_info,
                     print_success)


def move_files(
    source_paths: List[str],
    target_path: str,
    use_id: bool = False
):
    """移动文件到指定文件夹"""
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            # 创建路径解析器
            from ...services.name_resolver import NameResolver
            resolver = NameResolver(client.files)

            # 解析源文件路径或ID
            if use_id:
                file_ids = source_paths
                print_info(f"使用文件ID移动: {', '.join(file_ids)}")
            else:
                file_ids = []
                for path in source_paths:
                    try:
                        file_id, _ = resolver.resolve_path(path)
                        file_ids.append(file_id)
                        print_info(f"解析路径 '{path}' -> {file_id}")
                    except Exception as e:
                        print_error(f"无法解析路径 '{path}': {e}")
                        raise typer.Exit(1)

            # 解析目标文件夹路径或ID
            if use_id:
                target_folder_id = target_path
                print_info(f"目标文件夹ID: {target_folder_id}")
            else:
                try:
                    target_folder_id, target_type = resolver.resolve_path(target_path)

                    # 检查目标是否为文件夹
                    if target_type != 'folder':
                        print_error(f"目标路径不是文件夹: {target_path}")
                        raise typer.Exit(1)

                    print_info(f"目标文件夹: {target_path} -> {target_folder_id}")
                except Exception as e:
                    print_error(f"无法解析目标路径 '{target_path}': {e}")
                    raise typer.Exit(1)

            # 显示移动信息
            print_info("📦 开始移动文件...")
            print_info(f"   源文件数量: {len(file_ids)}")
            print_info(f"   目标文件夹: {target_path if not use_id else target_folder_id}")

            # 执行移动
            result = client.move_files(
                file_ids=file_ids,
                target_folder_id=target_folder_id
            )

            if result and result.get('status') == 200:
                data = result.get('data', {})
                task_id = data.get('task_id')
                finish = data.get('finish', False)

                if finish:
                    print_success("✅ 文件移动完成!")
                else:
                    print_success(f"✅ 文件移动完成! (任务ID: {task_id})")

                # 显示移动结果
                print_info(f"\n📊 移动结果:")
                print_info(f"   移动文件数: {len(file_ids)}")
                print_info(f"   状态: {'同步完成' if finish else '异步完成'}")

                if not use_id:
                    print_info(f"\n💡 提示: 文件已移动到 '{target_path}'")
            else:
                print_error("移动失败")
                raise typer.Exit(1)

    except Exception as e:
        handle_api_error(e, "移动文件")
        raise typer.Exit(1)


def move_to_folder(
    source_paths: List[str],
    folder_name: str,
    parent_folder: str = "/",
    create_folder: bool = True,
    use_id: bool = False
):
    """移动文件到指定名称的文件夹（如果不存在则创建）"""
    # TODO: 实现 use_id 参数功能
    _ = use_id  # 参数将在未来实现中使用
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            # 构造目标文件夹路径
            if parent_folder == "/":
                target_folder_path = f"/{folder_name}"
            else:
                target_folder_path = f"{parent_folder.rstrip('/')}/{folder_name}"

            print_info(f"目标文件夹路径: {target_folder_path}")

            # 初始化路径解析器
            try:
                from ...services.name_resolver import NameResolver
                resolver = NameResolver(client.files)
            except Exception as e:
                print_error(f"无法初始化路径解析器: {e}")
                raise typer.Exit(1)

            # 检查目标文件夹是否存在
            try:
                target_folder_id, _ = resolver.resolve_path(target_folder_path)
                print_info(f"找到现有文件夹: {target_folder_path}")
            except:
                if create_folder:
                    # 创建文件夹
                    print_info(f"创建新文件夹: {folder_name}")

                    # 解析父文件夹ID
                    if parent_folder == "/":
                        parent_folder_id = "0"
                    else:
                        try:
                            parent_folder_id, _ = resolver.resolve_path(parent_folder)
                        except Exception as e:
                            print_error(f"无法解析父文件夹路径 '{parent_folder}': {e}")
                            raise typer.Exit(1)

                    # 创建文件夹
                    create_result = client.create_folder(folder_name, parent_folder_id)
                    if create_result and create_result.get('status') == 200:
                        target_folder_id = create_result.get('data', {}).get('fid')
                        print_success(f"✅ 文件夹创建成功: {folder_name}")
                    else:
                        print_error(f"创建文件夹失败: {folder_name}")
                        raise typer.Exit(1)
                else:
                    print_error(f"目标文件夹不存在: {target_folder_path}")
                    print_info("使用 --create-folder 自动创建文件夹")
                    raise typer.Exit(1)

            # 调用移动函数
            move_files(source_paths, target_folder_id, use_id=True)

    except Exception as e:
        handle_api_error(e, "移动文件到文件夹")
        raise typer.Exit(1)



"""
分享相关命令
"""

from typing import List, Optional

import typer
from rich.console import Console
from rich.table import Table

from ..utils import (get_client, handle_api_error, print_error, print_info,
                     print_success, print_warning)


def create_share(
    file_paths: List[str],
    title: str = "",
    expire_days: int = 0,
    password: Optional[str] = None,
    use_id: bool = False
):
    """创建分享链接"""
    console = Console()

    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            # 解析文件路径或ID
            if use_id:
                file_ids = file_paths
                print_info(f"使用文件ID创建分享: {', '.join(file_ids)}")
            else:
                # 使用路径解析器将路径转换为ID
                from ...services.name_resolver import NameResolver
                resolver = NameResolver(client.files)

                file_ids = []
                for path in file_paths:
                    try:
                        file_id, _ = resolver.resolve_path(path)
                        file_ids.append(file_id)
                        print_info(f"解析路径 '{path}' -> {file_id}")
                    except Exception as e:
                        print_error(f"无法解析路径 '{path}': {e}")
                        raise typer.Exit(1)

            # 显示分享参数
            print_info("📤 创建分享链接...")
            if title:
                print_info(f"   标题: {title}")
            if expire_days > 0:
                print_info(f"   有效期: {expire_days} 天")
            else:
                print_info("   有效期: 永久")
            if password:
                print_info(f"   提取码: {password}")
            else:
                print_info("   提取码: 无")

            # 创建分享
            result = client.create_share(
                file_ids=file_ids,
                title=title,
                expire_days=expire_days,
                password=password
            )

            if result:
                print_success("✅ 分享创建成功!")

                # 显示分享信息
                table = Table(title="分享信息")
                table.add_column("属性", style="cyan")
                table.add_column("值", style="green")

                table.add_row("分享链接", result.get('share_url', 'N/A'))
                table.add_row("分享ID", result.get('pwd_id', 'N/A'))
                table.add_row("标题", result.get('title', 'N/A'))
                table.add_row("文件数量", str(result.get('file_num', 0)))

                if result.get('expired_type') == 1:
                    table.add_row("有效期", "永久")
                else:
                    expired_at = result.get('expired_at', 0)
                    if expired_at:
                        import datetime
                        expire_date = datetime.datetime.fromtimestamp(expired_at / 1000)
                        table.add_row("有效期", expire_date.strftime('%Y-%m-%d %H:%M:%S'))

                console.print(table)

                # 显示复制友好的格式
                share_url = result.get('share_url', '')
                if password:
                    print_info(f"\n📋 复制分享信息:")
                    print_info(f"链接: {share_url}")
                    print_info(f"提取码: {password}")
                else:
                    print_info(f"\n📋 分享链接: {share_url}")
            else:
                print_error("分享创建失败")
                raise typer.Exit(1)

    except Exception as e:
        handle_api_error(e, "创建分享")
        raise typer.Exit(1)


def list_my_shares(page: int = 1, size: int = 20):
    """列出我的分享"""
    console = Console()

    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            print_info(f"📋 获取我的分享列表 (第{page}页)...")

            result = client.get_my_shares(page=page, size=size)

            if result and result.get('status') == 200:
                data = result.get('data', {})
                shares = data.get('list', [])
                metadata = result.get('metadata', {})
                total = metadata.get('_total', 0)

                if not shares:
                    print_warning("暂无分享")
                    return

                print_success(f"✅ 找到 {total} 个分享")

                # 创建表格
                table = Table(title=f"我的分享 (第{page}页，共{total}个)")
                table.add_column("序号", style="cyan", width=4)
                table.add_column("标题", style="green", width=18)
                table.add_column("分享链接", style="bright_blue", width=35)
                table.add_column("类型", style="yellow", width=4)
                table.add_column("文件数", style="yellow", width=6)
                table.add_column("创建时间", style="blue", width=12)
                table.add_column("状态", style="magenta", width=6)
                table.add_column("访问量", style="dim", width=6)

                for i, share in enumerate(shares, 1):
                    # 格式化创建时间
                    created_at = share.get('created_at', 0)
                    if created_at:
                        import datetime
                        create_time = datetime.datetime.fromtimestamp(created_at / 1000)
                        time_str = create_time.strftime('%m-%d %H:%M')
                    else:
                        time_str = "未知"

                    # 状态
                    status = "正常" if share.get('status') == 1 else "已失效"

                    # 类型（文件夹或文件）
                    first_file = share.get('first_file', {})
                    is_dir = first_file.get('dir', False)
                    file_type = "📁" if is_dir else "📄"

                    # 分享链接（完整显示）
                    share_url = share.get('share_url', '')

                    # 访问量
                    click_pv = share.get('click_pv', 0)

                    table.add_row(
                        str(i),
                        share.get('title', '无标题')[:16],  # 稍微缩短标题
                        share_url,  # 完整显示分享链接
                        file_type,
                        str(share.get('file_num', 0)),
                        time_str,
                        status,
                        str(click_pv)
                    )

                console.print(table)

                # 显示详细统计信息
                print_info(f"\n📊 统计信息:")
                total_clicks = sum(share.get('click_pv', 0) for share in shares)
                total_saves = sum(share.get('save_pv', 0) for share in shares)
                total_downloads = sum(share.get('download_pv', 0) for share in shares)
                print_info(f"   总访问量: {total_clicks}")
                print_info(f"   总保存量: {total_saves}")
                print_info(f"   总下载量: {total_downloads}")

                # 分页信息
                total_pages = (total + size - 1) // size
                if total_pages > 1:
                    print_info(f"\n📄 第 {page}/{total_pages} 页")
                    if page < total_pages:
                        print_info(f"使用 'quarkpan shares --page {page + 1}' 查看下一页")
            else:
                print_error("获取分享列表失败")
                raise typer.Exit(1)

    except Exception as e:
        handle_api_error(e, "获取分享列表")
        raise typer.Exit(1)


def save_share(
    share_url: str,
    target_folder: str = "/",
    create_folder: bool = True
):
    """转存分享文件"""
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            print_info(f"🔗 解析分享链接: {share_url}")

            # 解析目标文件夹
            target_folder_id = "0"  # 默认根目录
            target_folder_name = None

            if target_folder != "/":
                # 解析目标文件夹路径
                from ...services.name_resolver import NameResolver
                resolver = NameResolver(client.files)

                try:
                    target_folder_id, _ = resolver.resolve_path(target_folder)
                    print_info(f"目标文件夹: {target_folder}")
                except:
                    if create_folder:
                        # 自动创建文件夹
                        target_folder_name = target_folder.split('/')[-1]
                        print_info(f"将创建新文件夹: {target_folder_name}")
                    else:
                        print_error(f"目标文件夹不存在: {target_folder}")
                        print_info("使用 --create-folder 自动创建文件夹")
                        raise typer.Exit(1)

            print_info("📥 开始转存...")

            result = client.save_shared_files(
                share_url=share_url,
                target_folder_id=target_folder_id,
                target_folder_name=target_folder_name
            )

            if result:
                share_info = result.get('share_info', {})
                file_count = share_info.get('file_count', 0)
                print_success(f"✅ 转存成功! 共转存 {file_count} 个文件")

                # 显示转存的文件信息
                files = share_info.get('files', [])
                if files and len(files) <= 10:  # 只显示前10个文件
                    print_info("\n📁 转存的文件:")
                    for file_info in files:
                        file_name = file_info.get('file_name', '未知文件')
                        file_size = file_info.get('size', 0)
                        if file_size > 0:
                            size_str = _format_size(file_size)
                            print_info(f"  📄 {file_name} ({size_str})")
                        else:
                            print_info(f"  📁 {file_name}")
            else:
                print_error("转存失败")
                raise typer.Exit(1)

    except Exception as e:
        handle_api_error(e, "转存分享")
        raise typer.Exit(1)


def _format_size(size: int) -> str:
    """格式化文件大小"""
    if size < 1024:
        return f"{size} B"
    elif size < 1024 * 1024:
        return f"{size / 1024:.1f} KB"
    elif size < 1024 * 1024 * 1024:
        return f"{size / (1024 * 1024):.1f} MB"
    else:
        return f"{size / (1024 * 1024 * 1024):.1f} GB"



"""
异常定义模块
"""

from typing import Optional


class QuarkClientError(Exception):
    """夸克客户端基础异常"""
    pass


class AuthenticationError(QuarkClientError):
    """认证相关异常"""
    pass


class ConfigError(QuarkClientError):
    """配置相关异常"""
    pass


class APIError(QuarkClientError):
    """API调用异常"""

    def __init__(self, message: str, status_code: Optional[int] = None, response_data: Optional[dict] = None):
        super().__init__(message)
        self.status_code = status_code
        self.response_data = response_data


class NetworkError(QuarkClientError):
    """网络相关异常"""
    pass


class FileNotFoundError(QuarkClientError):
    """文件未找到异常"""
    pass


class ShareLinkError(QuarkClientError):
    """分享链接相关异常"""
    pass


class DownloadError(QuarkClientError):
    """下载相关异常"""
    pass



"""
批量分享服务
"""

import csv
from datetime import datetime
from typing import Any, Dict, List, Optional, Tuple

from ..core.api_client import QuarkAPIClient
from ..exceptions import APIError
from ..utils.logger import get_logger
from .file_service import FileService
from .share_service import ShareService


class BatchShareService:
    """批量分享服务"""

    def __init__(self, client: QuarkAPIClient):
        """
        初始化批量分享服务

        Args:
            client: API客户端实例
        """
        self.client = client
        self.file_service = FileService(client)
        self.share_service = ShareService(client)
        self.logger = get_logger(__name__)

    def collect_target_directories(self, exclude_patterns: Optional[List[str]] = None) -> List[Dict[str, Any]]:
        """
        收集所有需要分享的目标目录（四级目录）

        Args:
            exclude_patterns: 排除的目录名称模式列表

        Returns:
            目标目录列表，每个元素包含目录信息和完整路径
        """
        if exclude_patterns is None:
            exclude_patterns = ["来自：分享"]

        target_directories = []

        self.logger.info("开始收集目标目录...")

        # 第一级：获取根目录下的所有文件夹（二级目录）
        try:
            root_response = self.file_service.list_files(folder_id="0", size=200)
            if not root_response.get('status') == 200:
                raise APIError("无法获取根目录文件列表")

            second_level_dirs = []
            root_files = root_response.get('data', {}).get('list', [])

            for item in root_files:
                if item.get('dir', False):  # 只处理文件夹
                    dir_name = item.get('file_name', '')
                    # 检查是否需要排除
                    if not any(pattern in dir_name for pattern in exclude_patterns):
                        second_level_dirs.append({
                            'fid': item.get('fid'),
                            'name': dir_name,
                            'path': f"/{dir_name}"
                        })
                        self.logger.info(f"找到二级目录: {dir_name}")
                    else:
                        self.logger.info(f"跳过排除目录: {dir_name}")

            # 第二级：遍历每个二级目录，获取三级目录
            for second_dir in second_level_dirs:
                try:
                    second_response = self.file_service.list_files(
                        folder_id=second_dir['fid'],
                        size=200
                    )
                    if not second_response.get('status') == 200:
                        self.logger.warning(f"无法获取二级目录文件列表: {second_dir['name']}")
                        continue

                    third_level_dirs = []
                    second_files = second_response.get('data', {}).get('list', [])

                    for item in second_files:
                        if item.get('dir', False):  # 只处理文件夹
                            dir_name = item.get('file_name', '')
                            third_level_dirs.append({
                                'fid': item.get('fid'),
                                'name': dir_name,
                                'path': f"{second_dir['path']}/{dir_name}"
                            })
                            self.logger.info(f"找到三级目录: {second_dir['name']}/{dir_name}")

                    # 第三级：遍历每个三级目录，获取四级目录（目标目录）
                    for third_dir in third_level_dirs:
                        try:
                            third_response = self.file_service.list_files(
                                folder_id=third_dir['fid'],
                                size=200
                            )
                            if not third_response.get('status') == 200:
                                self.logger.warning(f"无法获取三级目录文件列表: {third_dir['name']}")
                                continue

                            third_files = third_response.get('data', {}).get('list', [])

                            for item in third_files:
                                if item.get('dir', False):  # 只处理文件夹（目标目录）
                                    target_name = item.get('file_name', '')
                                    target_path = f"{third_dir['path']}/{target_name}"

                                    target_info = {
                                        'fid': item.get('fid'),
                                        'name': target_name,
                                        'full_path': target_path,
                                        'second_level': second_dir['name'],
                                        'third_level': third_dir['name'],
                                        'file_info': item
                                    }

                                    target_directories.append(target_info)
                                    self.logger.info(f"找到目标目录: {target_path}")

                        except Exception as e:
                            self.logger.error(f"处理三级目录时出错 {third_dir['name']}: {e}")
                            continue

                except Exception as e:
                    self.logger.error(f"处理二级目录时出错 {second_dir['name']}: {e}")
                    continue

        except Exception as e:
            self.logger.error(f"获取根目录时出错: {e}")
            raise

        self.logger.info(f"总共找到 {len(target_directories)} 个目标目录")
        return target_directories

    def create_batch_shares(self, target_directories: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
        """
        批量创建分享链接

        Args:
            target_directories: 目标目录列表

        Returns:
            分享结果列表
        """
        share_results = []
        total = len(target_directories)

        self.logger.info(f"开始批量创建分享，共 {total} 个目录")

        for i, target_dir in enumerate(target_directories, 1):
            try:
                self.logger.info(f"正在创建分享 ({i}/{total}): {target_dir['full_path']}")

                # 创建分享
                share_result = self.share_service.create_share(
                    file_ids=[target_dir['fid']],
                    title=target_dir['name'],  # 使用目录名作为分享标题
                    expire_days=0,  # 永久
                    password=None   # 无密码
                )

                if share_result:
                    # 添加额外信息到结果中
                    share_info = {
                        'target_directory': target_dir,
                        'share_result': share_result,
                        'share_title': target_dir['name'],
                        'share_url': share_result.get('share_url', ''),
                        'share_id': share_result.get('pwd_id', ''),
                        'created_time': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                        'success': True
                    }
                    share_results.append(share_info)
                    self.logger.info(f"分享创建成功: {target_dir['name']} -> {share_result.get('share_url', '')}")
                else:
                    # 分享失败
                    share_info = {
                        'target_directory': target_dir,
                        'share_result': None,
                        'share_title': target_dir['name'],
                        'share_url': '',
                        'share_id': '',
                        'created_time': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                        'success': False,
                        'error': '分享创建失败'
                    }
                    share_results.append(share_info)
                    self.logger.error(f"分享创建失败: {target_dir['name']}")

            except Exception as e:
                # 记录错误并继续
                share_info = {
                    'target_directory': target_dir,
                    'share_result': None,
                    'share_title': target_dir['name'],
                    'share_url': '',
                    'share_id': '',
                    'created_time': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                    'success': False,
                    'error': str(e)
                }
                share_results.append(share_info)
                self.logger.error(f"创建分享时出错 {target_dir['name']}: {e}")

        successful = sum(1 for result in share_results if result['success'])
        self.logger.info(f"批量分享完成: 成功 {successful}/{total}")

        return share_results

    def export_to_csv(self, share_results: List[Dict[str, Any]], filename: Optional[str] = None) -> str:
        """
        导出分享结果到CSV文件

        Args:
            share_results: 分享结果列表
            filename: CSV文件名，如果不指定则自动生成

        Returns:
            CSV文件路径
        """
        if filename is None:
            today = datetime.now().strftime('%Y-%m-%d')
            filename = f"shares_{today}.csv"

        # 确保文件名以.csv结尾
        if not filename.endswith('.csv'):
            filename += '.csv'

        try:
            with open(filename, 'w', newline='', encoding='utf-8') as csvfile:
                writer = csv.writer(csvfile)

                # 写入标题行
                headers = ['分享标题', '分享链接', '完整路径', '创建时间']
                writer.writerow(headers)

                # 写入数据行
                for result in share_results:
                    if result['success']:
                        row = [
                            result['share_title'],
                            result['share_url'],
                            result['target_directory']['full_path'],
                            result['created_time']
                        ]
                        writer.writerow(row)
                    else:
                        # 对于失败的分享，也记录到CSV中，但链接为空
                        row = [
                            result['share_title'],
                            f"创建失败: {result.get('error', '未知错误')}",
                            result['target_directory']['full_path'],
                            result['created_time']
                        ]
                        writer.writerow(row)

            self.logger.info(f"CSV文件已保存: {filename}")
            return filename

        except Exception as e:
            self.logger.error(f"保存CSV文件时出错: {e}")
            raise

    def batch_share_and_export(self, csv_filename: Optional[str] = None, exclude_patterns: Optional[List[str]] = None) -> Tuple[List[Dict[str, Any]], str]:
        """
        一站式批量分享和导出服务

        Args:
            csv_filename: CSV文件名
            exclude_patterns: 排除的目录名称模式列表

        Returns:
            (分享结果列表, CSV文件路径)
        """
        # 1. 收集目标目录
        self.logger.info("🔍 开始收集目标目录...")
        target_directories = self.collect_target_directories(exclude_patterns)

        if not target_directories:
            self.logger.warning("没有找到任何目标目录")
            return [], ""

        # 2. 批量创建分享
        self.logger.info("📤 开始批量创建分享...")
        share_results = self.create_batch_shares(target_directories)

        # 3. 导出到CSV
        self.logger.info("📊 开始导出CSV文件...")
        csv_path = self.export_to_csv(share_results, csv_filename)

        # 4. 统计信息
        successful = sum(1 for result in share_results if result['success'])
        failed = len(share_results) - successful

        self.logger.info(f"✅ 批量分享完成!")
        self.logger.info(f"   总计: {len(share_results)} 个目录")
        self.logger.info(f"   成功: {successful} 个")
        self.logger.info(f"   失败: {failed} 个")
        self.logger.info(f"   CSV文件: {csv_path}")

        return share_results, csv_path



# -*- coding: utf-8 -*-
"""
夸克网盘 Python 客户端

一个功能完整的夸克网盘API客户端，支持文件管理、分享转存等功能。
"""

# 初始化日志系统
from .utils.logger import setup_logger

setup_logger()

# 认证相关
from .auth.login import QuarkAuth, get_auth_cookies
# 主要客户端类
from .client import QuarkClient, create_client
# 核心API客户端
from .core.api_client import QuarkAPIClient
# 异常类
from .exceptions import (APIError, AuthenticationError, ConfigError,
                         DownloadError, FileNotFoundError, NetworkError,
                         QuarkClientError, ShareLinkError)
# 服务类
from .services.file_service import FileService
from .services.share_service import ShareService

__version__ = "0.1.0"
__author__ = "QuarkPan Team"
__email__ = "contact@quarkpan.dev"

__all__ = [
    # 主要客户端
    'QuarkClient',
    'create_client',

    # 认证
    'QuarkAuth',
    'get_auth_cookies',

    # 服务类
    'FileService',
    'ShareService',
    'QuarkAPIClient',

    # 异常
    'QuarkClientError',
    'AuthenticationError',
    'ConfigError',
    'APIError',
    'NetworkError',
    'FileNotFoundError',
    'ShareLinkError',
    'DownloadError',
]



"""
文件搜索命令
"""

from typing import List, Optional

import typer
from rich import print as rprint
from rich.console import Console
from rich.table import Table

from ..utils import (format_file_size, format_timestamp, get_client,
                     get_file_type_icon, handle_api_error, print_error,
                     print_info, print_warning, truncate_text)

search_app = typer.Typer(help="🔍 文件搜索")
console = Console()


@search_app.callback(invoke_without_command=True)
def search_main(
    ctx: typer.Context,
    keyword: Optional[str] = typer.Argument(None, help="搜索关键词"),
    folder_id: str = typer.Option("0", "--folder", "-f", help="搜索范围文件夹ID，默认全盘搜索"),
    page: int = typer.Option(1, "--page", "-p", help="页码"),
    size: int = typer.Option(20, "--size", "-s", help="每页数量"),
    show_details: bool = typer.Option(False, "--details", "-d", help="显示详细信息"),
    extensions: Optional[List[str]] = typer.Option(None, "--ext", "-e", help="文件扩展名过滤 (如: pdf, doc, mp4)"),
    min_size: Optional[str] = typer.Option(None, "--min-size", help="最小文件大小 (如: 1MB, 100KB)"),
    max_size: Optional[str] = typer.Option(None, "--max-size", help="最大文件大小 (如: 100MB, 1GB)")
):
    """搜索文件"""
    if ctx.invoked_subcommand is not None:
        return

    if not keyword:
        rprint("[red]❌ 请提供搜索关键词[/red]")
        rprint("使用: [cyan]quarkpan search \"关键词\"[/cyan]")
        raise typer.Exit(1)

    # 直接在这里实现搜索逻辑
    do_search(keyword, folder_id, page, size, show_details, extensions, min_size, max_size)


def do_search(
    keyword: str,
    folder_id: str = "0",
    page: int = 1,
    size: int = 20,
    show_details: bool = False,
    file_extensions: Optional[List[str]] = None,
    min_size: Optional[str] = None,
    max_size: Optional[str] = None
):
    """搜索文件"""
    try:
        with get_client() as client:
            if not client.is_logged_in():
                print_error("未登录，请先使用 quarkpan auth login 登录")
                raise typer.Exit(1)

            print_info(f"正在搜索: {keyword}")

            # 解析文件大小
            min_size_bytes = parse_file_size(min_size) if min_size else None
            max_size_bytes = parse_file_size(max_size) if max_size else None

            # 执行搜索
            if file_extensions or min_size_bytes or max_size_bytes:
                # 高级搜索
                results = client.search_files_advanced(
                    keyword=keyword,
                    folder_id=folder_id,
                    page=page,
                    size=size,
                    file_extensions=file_extensions,
                    min_size=min_size_bytes,
                    max_size=max_size_bytes
                )
            else:
                # 基础搜索
                results = client.search_files(
                    keyword=keyword,
                    folder_id=folder_id,
                    page=page,
                    size=size
                )

            if not results or 'data' not in results:
                print_error("搜索失败")
                raise typer.Exit(1)

            file_list = results['data'].get('list', [])
            # 搜索API的total在metadata中
            total = results.get('metadata', {}).get('_total', len(file_list))

            # 显示搜索结果
            search_scope = "全盘" if folder_id == "0" else f"文件夹 {folder_id}"
            rprint(f"\n🔍 搜索结果: [bold]{keyword}[/bold] (范围: {search_scope})")

            if not file_list:
                print_warning("没有找到匹配的文件")
                return

            # 显示过滤条件
            filters = []
            if file_extensions:
                filters.append(f"扩展名: {', '.join(file_extensions)}")
            if min_size:
                filters.append(f"最小: {min_size}")
            if max_size:
                filters.append(f"最大: {max_size}")

            if filters:
                rprint(f"[dim]过滤条件: {' | '.join(filters)}[/dim]")

            if show_details:
                # 详细表格视图
                table = Table(title=f"第{page}页，共{total}个结果")
                table.add_column("序号", style="dim", width=4)
                table.add_column("类型", style="cyan", width=4)
                table.add_column("名称", style="white", min_width=25)
                table.add_column("大小", style="green", width=10)
                table.add_column("修改时间", style="yellow", width=16)
                table.add_column("ID", style="dim", width=8)

                for i, file_info in enumerate(file_list, (page - 1) * size + 1):
                    name = file_info.get('file_name', '未知')
                    size_bytes = file_info.get('size', 0)
                    file_type = file_info.get('file_type', 0)
                    updated_at = file_info.get('updated_at', '')
                    fid = file_info.get('fid', '')

                    is_folder = file_type == 0
                    type_icon = get_file_type_icon(name, is_folder)
                    size_str = "-" if is_folder else format_file_size(size_bytes)
                    time_str = format_timestamp(updated_at) if updated_at else "-"
                    short_id = fid[:8] + "..." if len(fid) > 8 else fid

                    table.add_row(
                        str(i),
                        type_icon,
                        truncate_text(name, 30),
                        size_str,
                        time_str,
                        short_id
                    )

                console.print(table)
            else:
                # 简洁列表视图
                rprint(f"[dim]第{page}页，共{total}个结果[/dim]\n")

                for i, file_info in enumerate(file_list, (page - 1) * size + 1):
                    name = file_info.get('file_name', '未知')
                    file_type = file_info.get('file_type', 0)
                    size_bytes = file_info.get('size', 0)

                    is_folder = file_type == 0
                    type_icon = get_file_type_icon(name, is_folder)

                    if is_folder:
                        rprint(f"  {i:2d}. {type_icon} {name}")
                    else:
                        size_str = format_file_size(size_bytes)
                        rprint(f"  {i:2d}. {type_icon} {name} [dim]({size_str})[/dim]")

            # 显示分页信息
            if total > size:
                total_pages = (total + size - 1) // size
                rprint(f"\n[dim]第 {page}/{total_pages} 页，共 {total} 个结果[/dim]")
                if page < total_pages:
                    rprint(f"[dim]使用 --page {page + 1} 查看下一页[/dim]")

    except Exception as e:
        handle_api_error(e, "搜索文件")
        raise typer.Exit(1)


def parse_file_size(size_str: str) -> Optional[int]:
    """解析文件大小字符串，返回字节数"""
    if not size_str:
        return None

    size_str = size_str.upper().strip()

    # 提取数字和单位
    import re
    match = re.match(r'^(\d+(?:\.\d+)?)\s*([KMGT]?B?)$', size_str)
    if not match:
        return None

    number = float(match.group(1))
    unit = match.group(2) or 'B'

    # 转换为字节
    multipliers = {
        'B': 1,
        'KB': 1024,
        'MB': 1024 ** 2,
        'GB': 1024 ** 3,
        'TB': 1024 ** 4,
        'K': 1024,
        'M': 1024 ** 2,
        'G': 1024 ** 3,
        'T': 1024 ** 4
    }

    return int(number * multipliers.get(unit, 1))


if __name__ == "__main__":
    search_app()
